﻿Imports FeedCast.ViewModels
Imports FeedCastLibrary

Partial Public Class App
    Inherits Application

    Public Shared ReadOnly DBLocation As String = "Data Source=isostore:/LocalDatabase.sdf"

    Private Shared _dataBaseUtility As DataUtils
    Public Shared Property DataBaseUtility As DataUtils
        Get
            Return _dataBaseUtility
        End Get
        Private Set(ByVal value As DataUtils)
            _dataBaseUtility = value
        End Set
    End Property

    Private Shared _applicationSettings As Settings
    Public Shared Property ApplicationSettings As Settings
        Get
            Return _applicationSettings
        End Get
        Private Set(ByVal value As Settings)
            _applicationSettings = value
        End Set
    End Property

    Public Shared Property SynFeedDownloader As WebTools

    Public Shared Property Favorites As Category

    Public Shared Property ArticleSource As String()

    ''' <summary>
    ''' Provides easy access to the root frame of the Phone Application.
    ''' </summary>
    ''' <returns>The root frame of the Phone Application.</returns>
    Public Property RootFrame As PhoneApplicationFrame

    ''' <summary>
    ''' Constructor for the Application object.
    ''' </summary>
    Public Sub New()
        ' Standard Silverlight initialization
        InitializeComponent()

        ' Phone-specific initialization
        InitializePhoneApplication()

        ' Enable URI Mapper
        RootFrame.UriMapper = TryCast(Resources("UriMapper"), UriMapper)

        ' Show graphics profiling information while debugging.
        If System.Diagnostics.Debugger.IsAttached Then
            ' Display the current frame rate counters.
            Application.Current.Host.Settings.EnableFrameRateCounter = False

            ' Show the areas of the app that are being redrawn in each frame.
            'Application.Current.Host.Settings.EnableRedrawRegions = True

            ' Enable non-production analysis visualization mode, 
            ' which shows areas of a page that are handed off to GPU with a colored overlay.
            'Application.Current.Host.Settings.EnableCacheVisualization = True

            ' Disable the application idle detection by setting the UserIdleDetectionMode property of the
            ' application's PhoneApplicationService object to Disabled.
            ' Caution:- Use this under debug mode only. Application that disables user idle detection will continue to run
            ' and consume battery power when the user is not using the phone.
            PhoneApplicationService.Current.UserIdleDetectionMode = IdleDetectionMode.Disabled
        End If

        ' Initialize application wide settings and DataBase utilities.
        Using db As New LocalDatabaseDataContext(DBLocation)
            'Inorder to clean the db uncomment the next line (or, if debugging, rebuild solution)
            'db.DeleteDatabase
            If Not db.DatabaseExists Then
                db.CreateDatabase()
            End If
            db.SubmitChanges()
        End Using

        ' Needs to be called AFTER DatabaseContext is Disposed above.
        DataBaseUtility = New DataUtils(DBLocation)

        Favorites = New Category With {.CategoryTitle = "Favorites"}
        App.DataBaseUtility.AddCategory(Favorites)
        App.DataBaseUtility.SaveChangesToDB()

        ApplicationSettings = New Settings

        ' WebTools object used for initial download and refreshing.
        SynFeedDownloader = New WebTools(New SynFeedParser)

        ' ArticleSource array used to keep track of article page source.
        ArticleSource = New String(1) {}

        ' Create the background agent if it does not already exist on the scheduler.
        Dim BGTools As New BackgroundAgentTools
        BGTools.StartPeriodicAgent()

        ' Redirect initial navigation.
        AddHandler RootFrame.Navigating, AddressOf RootFrame_Navigating
    End Sub

#Region "Initial Redirection"
    Private Sub RootFrame_Navigating(ByVal sender As Object, ByVal e As NavigatingCancelEventArgs)
        'Check for initial navigation
        If Not e.Uri.ToString.Contains("/MainPage.xaml") Then
            Exit Sub
        Else
            If Settings.InitialLaunchSetting Then
                'Cancel previous navigation
                e.Cancel = True
                'Sync new navigation
                RootFrame.Dispatcher.BeginInvoke(Sub() RootFrame.Navigate(New Uri("/Views/LaunchPage.xaml", UriKind.Relative)))
            End If
        End If
    End Sub
#End Region

    ' Code to execute when the application is launching (eg, from Start)
    ' This code will not execute when the application is reactivated
    Private Sub Application_Launching(ByVal sender As Object, ByVal e As LaunchingEventArgs)
    End Sub

    ' Code to execute when the application is activated (brought to foreground)
    ' This code will not execute when the application is first launched
    Private Sub Application_Activated(ByVal sender As Object, ByVal e As ActivatedEventArgs)
    End Sub

    ' Code to execute when the application is deactivated (sent to background)
    ' This code will not execute when the application is closing
    Private Sub Application_Deactivated(ByVal sender As Object, ByVal e As DeactivatedEventArgs)
    End Sub

    ' Code to execute when the application is closing (eg, user hit Back)
    ' This code will not execute when the application is deactivated
    Private Sub Application_Closing(ByVal sender As Object, ByVal e As ClosingEventArgs)
    End Sub

    ' Code to execute if a navigation fails
    Private Sub RootFrame_NavigationFailed(ByVal sender As Object, ByVal e As NavigationFailedEventArgs)
        If Diagnostics.Debugger.IsAttached Then
            ' A navigation has failed; break into the debugger
            Diagnostics.Debugger.Break()
        End If
    End Sub

    Public Sub Application_UnhandledException(ByVal sender As Object, ByVal e As ApplicationUnhandledExceptionEventArgs) Handles Me.UnhandledException
        ' Show graphics profiling information while debugging.
        If Diagnostics.Debugger.IsAttached Then
            Diagnostics.Debugger.Break()
        Else
            e.Handled = True
            MessageBox.Show(e.ExceptionObject.Message & Environment.NewLine & e.ExceptionObject.StackTrace,
                            "Error", MessageBoxButton.OK)
        End If
    End Sub

#Region "Phone application initialization"
    ' Avoid double-initialization
    Private phoneApplicationInitialized As Boolean = False

    ' Do not add any additional code to this method
    Private Sub InitializePhoneApplication()
        If phoneApplicationInitialized Then
            Exit Sub
        End If

        ' Create the frame but don't set it as RootVisual yet; this allows the splash
        ' screen to remain active until the application is ready to render.
        RootFrame = New PhoneApplicationFrame
        AddHandler RootFrame.Navigated, AddressOf CompleteInitializePhoneApplication

        ' Handle navigation failures
        AddHandler RootFrame.NavigationFailed, AddressOf RootFrame_NavigationFailed

        ' Ensure we don't initialize again
        phoneApplicationInitialized = True
    End Sub

    ' Do not add any additional code to this method
    Private Sub CompleteInitializePhoneApplication(ByVal sender As Object, ByVal e As NavigationEventArgs)
        ' Set the root visual to allow the application to render
        If RootVisual IsNot RootFrame Then
            RootVisual = RootFrame
        End If

        ' Remove this handler since it is no longer needed
        RemoveHandler RootFrame.Navigated, AddressOf CompleteInitializePhoneApplication
    End Sub
#End Region

End Class