﻿' 
'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  
'

Imports System.Collections.ObjectModel
Imports FeedCastLibrary

Namespace Models
    Public NotInheritable Class FeaturedAlgorithm
        ''' <summary>
        ''' Determines which articles will be placed into the featured section. Updates the stored featured articles.
        ''' </summary>
        Private Sub New()
        End Sub

        Public Shared Sub UpdateFeatured()
            Dim articles As New Collection(Of Article)
            Dim favConst As Integer = 10
            Dim shareConst As Integer = 30

            ' Query the database for feeds with unread articles.
            Dim unreadFeeds As List(Of Feed) = App.DataBaseUtility.QueryForUnreadFeeds

            Dim thresholds(unreadFeeds.Count - 1) As Integer

            ' Calculate the threshold value for each of the feeds. Top threshold feeds are chosen.
            For i = 0 To unreadFeeds.Count - 1
                ' c.Skip(i).Take(1).Single is essentially the equivalent of c.ElementAt(i).
                ' c.ElementAt(i) is not an allowed operation for QueriableItem c.
                Dim feed As Feed = unreadFeeds(i)

                ' Give threshold a minimum value of 1 to allow it to work well with MaxIndex().
                thresholds(i) = 1 + feed.ViewCount.Value + (feed.FavoritedCount.Value * favConst) + (feed.SharedCount.Value * shareConst)
            Next i

            ' Take the maximum 6 threshold indexes, which will be used to obtain the articles.
            Dim maxThresIndexes(5) As Integer
            Dim articleCount As Integer = 0 ' 6 articles are necessary for the Featured section.
            Dim numOfFeeds As Integer = 0 ' Number of feeds it takes to obtain 6 articles.
            Do While numOfFeeds < 6
                maxThresIndexes(numOfFeeds) = MaxIndex(thresholds)

                ' In case there are no more unread feeds:
                If maxThresIndexes(numOfFeeds) = -1 Then
                    Exit Do
                End If

                articleCount += unreadFeeds(maxThresIndexes(numOfFeeds)).UnreadCount.Value
                numOfFeeds += 1
            Loop

            If articleCount >= 6 Then
                ' There are enough new articles; update.
                ' Article getting process.
                Dim index As Integer = 0
                Do While articles.Count < 6
                    If index >= numOfFeeds Then
                        index = 0
                    End If

                    ' Query for the newest article of the chosen feed.
                    Dim sortedFeed As List(Of Article) = App.DataBaseUtility.QueryForSortedFeed(unreadFeeds, maxThresIndexes, index)

                    ' Add the article, and temporarily change it to read so it won't be added again.
                    ' Check to ensure that the query was successful, and unread articles do exist.
                    If sortedFeed.Count = 0 Then
                        Exit Do
                    Else
                        Dim newestArticle As Article = sortedFeed(0)
                        articles.Add(newestArticle)
                        newestArticle.Read = True
                        App.DataBaseUtility.SaveChangesToDB()
                    End If

                    index += 1
                Loop
            End If

            ' Return the read values in the articles to unread.
            ReturnArticlesToUnread(articles)

            ' Update the featured articles if successful (as in, 6 articles have been added).
            If articles.Count = 6 Then
                Settings.FeaturedArticles = articles
            End If
            Exit Sub
        End Sub

        ''' <summary>
        ''' Takes the index of the maximum int value in the array.
        ''' </summary>
        ''' <param name="numbers">The array of numbers to be looked at</param>
        ''' <returns>The index of the maximum int value in the array</returns>
        Private Shared Function MaxIndex(ByVal numbers() As Integer) As Integer
            ' Empty case.
            If numbers.Length = 0 Then
                Return -1
            End If
            Dim max As Integer = 0
            For i = 0 To numbers.Length - 1
                If numbers(i) > numbers(max) Then
                    max = i
                End If
            Next i

            ' Check for the no-max case.
            If numbers(max) = -1 Then
                Return -1
            Else
                numbers(max) = -1
                Return max
            End If
        End Function

        ''' <summary>
        ''' Return the iterated-over articles to unread.
        ''' </summary>
        ''' <param name="articles">The collection of articles to be changed</param>
        Private Shared Sub ReturnArticlesToUnread(ByVal articles As Collection(Of Article))
            For i = 0 To articles.Count - 1
                articles(i).Read = False
            Next i
            App.DataBaseUtility.SaveChangesToDB()
        End Sub
    End Class
End Namespace