﻿' 
'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  
'

Namespace Models
    ''' <summary>
    ''' A ListBox with Header and SubHeader scrolling content.
    ''' </summary>
    <TemplatePartAttribute(Name:="Header", Type:=GetType(TextBlock)), TemplatePartAttribute(Name:="LastUpdated", Type:=GetType(TextBlock))>
    Public Class HeaderedListBox
        Inherits ListBox

#Region "TemplatePart Names"
        ''' <summary>
        ''' Template Part name for Header TextBlock.
        ''' </summary>
        Private Const HeaderName As String = "Header"

        ''' <summary>
        ''' Template Part name for SubHeader TextBlock.
        ''' </summary>
        Private Const LastUpdatedName As String = "LastUpdated"
#End Region

#Region "Private fields"
        ''' <summary>
        ''' Header TextBlock.
        ''' </summary>
        Private _header As TextBlock

        ''' <summary>
        ''' SubHeader TextBlock.
        ''' </summary>
        Private _lastUpdatedText As TextBlock
#End Region

#Region "Dependency Properties"
#Region "HeaderText"
        ''' <summary>
        ''' The Header text for this HeaderedListBox.
        ''' </summary>
        Public Property HeaderText As String
            Get
                Return CStr(GetValue(HeaderTextProperty))
            End Get
            Set(ByVal value As String)
                _header.Visibility = If(String.IsNullOrEmpty(value) AndAlso Nothing IsNot _header, Visibility.Collapsed, Visibility.Visible)
                SetValue(HeaderTextProperty, value)
            End Set
        End Property

        ''' <summary>
        ''' Property for the Header text for this HeaderedListBox.
        ''' </summary>
        Public Shared ReadOnly HeaderTextProperty As DependencyProperty = DependencyProperty.Register("HeaderText", GetType(String), GetType(HeaderedListBox), New PropertyMetadata(Nothing))
#End Region

#Region "LastUpdatedText"
        ''' <summary>
        ''' The SubHeader text for this HeaderedListBox.
        ''' </summary>
        Public Property LastUpdatedText As String
            Get
                Return CStr(GetValue(LastUpdatedTextProperty))
            End Get
            Set(ByVal value As String)
                _lastUpdatedText.Visibility = If(String.IsNullOrEmpty(value) AndAlso Nothing IsNot _lastUpdatedText, Visibility.Collapsed, Visibility.Visible)
                SetValue(LastUpdatedTextProperty, value)
            End Set
        End Property

        ''' <summary>
        ''' Property for the SubHeader text for this HeaderedListBox.
        ''' </summary>
        Public Shared ReadOnly LastUpdatedTextProperty As DependencyProperty = DependencyProperty.Register("LastUpdatedText", GetType(String), GetType(HeaderedListBox), New PropertyMetadata("Last updated 1 minute ago."))
#End Region
#End Region

        ''' <summary>
        ''' Instantiates a new HeaderedListBox.
        ''' </summary>
        Public Sub New()
            MyBase.New()
            Me.DefaultStyleKey = GetType(HeaderedListBox)
        End Sub

        Public Overrides Sub OnApplyTemplate()
            MyBase.OnApplyTemplate()

            _header = TryCast(GetTemplateChild(HeaderName), TextBlock)
            _lastUpdatedText = TryCast(GetTemplateChild(LastUpdatedName), TextBlock)
        End Sub
    End Class
End Namespace