﻿' 
'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  
'

Imports System.Text
Imports FeedCastLibrary

Namespace Models
    ''' <summary>
    ''' Represents a category in the launch page used to initially populate the feed reader.
    ''' </summary>
    Public Class InitialCategory
        Inherits Category

        ''' <summary>
        ''' A comma-separated string of the initial feeds to be loaded with this InitialCategory.
        ''' </summary>
        Private _associatedFeeds As String
        Public Property AssociatedFeeds As String
            Get
                Return _associatedFeeds
            End Get
            Private Set(ByVal value As String)
                _associatedFeeds = value
            End Set
        End Property

        ''' <summary>
        ''' A list of the feeds to be loaded with this InitialCategory.
        ''' </summary>
        Private _feeds As IList(Of Feed)
        Public Property Feeds As IList(Of Feed)
            Get
                Return _feeds
            End Get
            Private Set(ByVal value As IList(Of Feed))
                _feeds = value
            End Set
        End Property

        ''' <summary>
        ''' Constructs an InitialCategory with the provided parameters.
        ''' </summary>
        ''' <param name="title">The title of this InitialCategory.</param>
        ''' <param name="feeds">One or more feeds to load when this InitialCategory is loaded.</param>
        Public Sub New(ByVal title As String, ByVal ParamArray feeds() As Feed)
            MyBase.New()
            CategoryTitle = title
            Me.Feeds = New List(Of Feed)(feeds)

            ' Building up AssociatedFeeds.
            Dim sb As New StringBuilder
            Dim separator As String = ", "
            For Each f In feeds
                sb.Append(f.FeedTitle).Append(separator)
            Next f
            sb.Length -= separator.Length
            AssociatedFeeds = sb.ToString
        End Sub
    End Class
End Namespace