﻿Imports FeedCastLibrary

Namespace ViewModels
    Public NotInheritable Class AddTile
        ''' <summary>
        ''' Create a secondary tile linking to an existing feed.
        '''  <param name="feed">The feed to be linked to on the tile</param> 
        ''' </summary>
        Private Sub New()
        End Sub

        Public Shared Sub AddLiveTile(ByVal feed As Feed)
            ' Look to see whether the Tile already exists; if so, don't try to create it again.
            Dim tileToFind As ShellTile = ShellTile.ActiveTiles.FirstOrDefault(Function(x) x.NavigationUri.ToString.Contains("/Feed/" & feed.FeedID.ToString))

            ' Create the Tile if we didn't find that it already exists.
            If tileToFind Is Nothing Then
                ' Create the Tile object and set some initial properties for the Tile.
                Dim newTileData As StandardTileData = New StandardTileData With {.BackgroundImage = New Uri(feed.ImageURL, UriKind.RelativeOrAbsolute), .Title = feed.FeedTitle, .Count = 0, .BackTitle = feed.FeedTitle, .BackContent = "Read the latest in " & feed.FeedTitle & "!"}

                ' Create the Tile and pin it to Start. This will cause a navigation to Start and a deactivation of our application.
                ShellTile.Create(New Uri("/Feed/" & feed.FeedID, UriKind.Relative), newTileData)
                ' note: Tile URI could also have listed the full path to the feed page. For example --> /Views/FeedPage.xaml?id=

                feed.IsPinned = True
                App.DataBaseUtility.SaveChangesToDB()
            End If
        End Sub

        Public Shared Sub AddLiveTile(ByVal cat As Category)
            ' Look to see whether the Tile already exists; if so, don't try to create it again.
            Dim tileToFind As ShellTile = ShellTile.ActiveTiles.FirstOrDefault(Function(x) x.NavigationUri.ToString.Contains("/Category/" & cat.CategoryID.ToString))

            ' Create the Tile if we didn't find that it already exists.
            If tileToFind Is Nothing Then
                ' Create an image for the category if there isnt one. 
                If cat.ImageURL Is Nothing OrElse cat.ImageURL = String.Empty Then
                    cat.ImageURL = ImageGrabber.GetDefaultImage
                End If

                ' Create the Tile object and set some initial properties for the Tile.
                Dim newTileData As StandardTileData = New StandardTileData With {.BackgroundImage = New Uri(cat.ImageURL, UriKind.RelativeOrAbsolute), .Title = cat.CategoryTitle, .Count = 0, .BackTitle = cat.CategoryTitle, .BackContent = "Read the latest in " & cat.CategoryTitle & "!"}

                ' Create the Tile and pin it to Start. This will cause a navigation to Start and a deactivation of our application.
                ShellTile.Create(New Uri("/Category/" & cat.CategoryID, UriKind.Relative), newTileData)
                ' note: Tile URI could have listed the full path to the Category page. For example --> /Views/CategoryPage.xaml?id=

                cat.IsPinned = True
                App.DataBaseUtility.SaveChangesToDB()
            End If
        End Sub
    End Class
End Namespace