﻿' 
'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  
' Enables user-specified timing of the background agent for debugging.
'#define DEBUG_AGENT

Imports Microsoft.Phone.Scheduler

Namespace ViewModels
    Public Class BackgroundAgentTools
        Private periodicDownload As PeriodicTask
        Private periodicTaskName As String

        ''' <summary>
        ''' Constructor
        ''' </summary>
        Public Sub New()
            periodicTaskName = "FeedCastAgent"
        End Sub

        ''' <summary>
        ''' Creates or renews the periodic agent on the scheduler.
        ''' </summary>
        ''' <returns>Whether the periodic agent is created or not</returns>
        Public Function StartPeriodicAgent() As Boolean
            periodicDownload = TryCast(ScheduledActionService.Find(periodicTaskName), PeriodicTask)
            Dim wasAdded As Boolean = True

            ' If the task already exists and the IsEnabled property is false, then background
            ' agents have been disabled by the user.
            If periodicDownload IsNot Nothing AndAlso (Not periodicDownload.IsEnabled) Then
                ' Can't add the agent. Return false!
                wasAdded = False
            End If

            ' If the task already exists and background agents are enabled for the
            ' application, then remove the agent and add again to update the scheduler.
            If periodicDownload IsNot Nothing AndAlso periodicDownload.IsEnabled Then
                ScheduledActionService.Remove(periodicTaskName)
            End If

            periodicDownload = New PeriodicTask(periodicTaskName)
            periodicDownload.Description = "Allows FeedCast to download new articles on a regular schedule."

            ' Scheduling the agent may not be allowed because maximum number 
            ' of agents has been reached or the phone is a 256-MB device.
            Try
                ScheduledActionService.Add(periodicDownload)
            Catch e1 As SchedulerServiceException
            End Try

            ' If debugging is enabled, use LaunchForTest to launch the agent in one minute.
            '#If(DEBUG_AGENT) Then
            ' ScheduledActionService.LaunchForTest(periodicTaskName, TimeSpan.FromSeconds(60))
            '#End If

            Return wasAdded
        End Function
    End Class
End Namespace