﻿' 
'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  
'
Imports FeedCastLibrary
Imports System.Collections.Generic

Namespace ViewModels
    Public Class ContentLoader
        Private Shared Downloader As New WebTools(New SynFeedParser)
        Private _lock As New Object
        Public _callback As Action

        ''' <summary>
        ''' Default constructor
        ''' </summary>
        Public Sub New()
        End Sub

        ''' <summary>
        ''' Constructor with a callback action
        ''' </summary>
        ''' <param name="callback"></param>
        Public Sub New(ByVal callback As Action)
            _callback = callback
        End Sub

        ''' <summary>
        ''' Event raised after all feed downloads have finished.
        ''' </summary>
        Public Event LoadingFinished As EventHandler(Of LoadingFinishedEventArgs)

        Public Sub DownloadFeeds(ByVal feeds As IList(Of Feed))
            If feeds.Count <> 0 Then
                If Not Downloader.IsDownloading Then
                    AddHandler Downloader.AllDownloadsFinished, AddressOf SaveToDB
                    Downloader.Download(feeds)
                End If
            Else
                Deployment.Current.Dispatcher.BeginInvoke(Sub() _callback())
            End If
        End Sub

        Private Sub SaveToDB(ByVal sender As Object, ByVal e As AllDownloadsFinishedEventArgs)
            ' For every collection in the dictionary returned.
            For Each feed In e.Downloads.Keys
                ' Make sure its not null!!
                If Nothing IsNot feed Then
                    Dim articles As ICollection(Of Article) = e.Downloads(feed)
                    If Nothing IsNot articles AndAlso articles.Count > 0 Then
                        ' Add it to the database.
                        App.DataBaseUtility.AddArticles(articles, feed)
                    End If
                End If
            Next feed

            App.DataBaseUtility.clearOldArticles()

            If Nothing IsNot LoadingFinishedEvent Then
                Settings.LastUpdatedTime = Date.Now.ToString
                RaiseEvent LoadingFinished(Me, New LoadingFinishedEventArgs)
            End If
        End Sub

        ''' <summary>
        ''' Event arguments passed when all Feed downloads have finished.
        ''' </summary>
        Public Class LoadingFinishedEventArgs
            Inherits EventArgs

            ''' <summary>
            ''' Creates a new instance of LoadingFinishedEventArgs.
            ''' </summary>
            Public Sub New()
                MyBase.New()
            End Sub

        End Class
    End Class
End Namespace