﻿' 
'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  
'
Imports System.Collections
Imports System.Collections.Generic
Imports System.Collections.ObjectModel
Imports FeedCast.Models
Imports FeedCastLibrary
Imports FeedCast.Views
Imports FeedCast.Resources

Namespace ViewModels
    ''' <summary>
    ''' This is the ViewModel for the Initial Launch of the application.
    ''' The user will be presented with the Loaded Initial Catgories to choose from.
    ''' </summary>
    Public Class LaunchPageViewModel
        Inherits ObservableCollection(Of InitialCategory)

        Public Property LoadContent As ContentLoader

        ''' <summary>
        ''' Creates a new instance of LaunchPageViewModel with the given NavigationService.
        '''  <param name="navService">The NavigationService used </param>
        ''' <param name="FinishedUri">The Uri to navigate to when this ViewModel
        ''' has finished downloading and storing all the initial categories.</param>
        ''' </summary>
        Public Sub New()
            LoadContent = New ContentLoader
            LoadInitialCategories()
        End Sub

        ''' <summary>
        ''' Loads the defined InitialCategories to memory.
        ''' </summary>
        Public Sub LoadInitialCategories()
            ' Add InitialCategories here with Feeds (Must contain valid FeedBaseURI's). 
            Add(New InitialCategory(AppResources.BusinessCategoryTitleText, New Feed With {.FeedTitle = "CNN Money", .FeedBaseURI = "http://rss.cnn.com/rss/money_topstories.rss"}, New Feed With {.FeedTitle = "Fox Business", .FeedBaseURI = "http://feeds.foxbusiness.com/foxbusiness/latest"}))
            Add(New InitialCategory(AppResources.EntertainmentCategoryTitleText, New Feed With {.FeedTitle = "Hollywood Reporter", .FeedBaseURI = "http://feeds.feedburner.com/thr/news"}, New Feed With {.FeedTitle = "Huffington Post Entertainment", .FeedBaseURI = "http://www.huffingtonpost.com/feeds/verticals/entertainment/index.xml"}))
            Add(New InitialCategory(AppResources.FashionCategoryTitleText, New Feed With {.FeedTitle = "Style", .FeedBaseURI = "http://www.style.com/homepage/rss"}, New Feed With {.FeedTitle = "Us Magazine", .FeedBaseURI = "http://www.usmagazine.com/celebrity_news/rss"}))
            Add(New InitialCategory(AppResources.HealthCategoryTitleText, New Feed With {.FeedTitle = "Huffington Post Health", .FeedBaseURI = "http://www.huffingtonpost.com/feeds/verticals/health/index.xml"}, New Feed With {.FeedTitle = "WebMD", .FeedBaseURI = "http://rssfeeds.webmd.com/rss/rss.aspx?RSSSource=RSS_PUBLIC"}))
            Add(New InitialCategory(AppResources.MusicCategoryTitleText, New Feed With {.FeedTitle = "VH1", .FeedBaseURI = "http://www.bestweekever.tv/feed/"}, New Feed With {.FeedTitle = "MTV", .FeedBaseURI = "http://www.mtv.com/rss/news/news_full.jhtml"}))
            Add(New InitialCategory(AppResources.NewsCategoryTitleText, New Feed With {.FeedTitle = "BBC", .FeedBaseURI = "http://feeds.bbci.co.uk/news/rss.xml"}, New Feed With {.FeedTitle = "CNN", .FeedBaseURI = "http://rss.cnn.com/rss/cnn_topstories.rss"}))
            Add(New InitialCategory(AppResources.PoliticsCategoryTitleText, New Feed With {.FeedTitle = "NY Times", .FeedBaseURI = "http://www.nytimes.com/services/xml/rss/nyt/GlobalHome.xml"}, New Feed With {.FeedTitle = "Reuters", .FeedBaseURI = "http://www.reuters.com/rssFeed/topNews"}))

            Add(New InitialCategory(AppResources.SportsCategoryTitleText, New Feed With {.FeedTitle = "ESPN", .FeedBaseURI = "http://sports.espn.go.com/espn/rss/news"}, New Feed With {.FeedTitle = "Yahoo Sports", .FeedBaseURI = "http://sports.yahoo.com/sow/rss.xml"}))
            Add(New InitialCategory(AppResources.TechnologyCategoryTitleText, New Feed With {.FeedTitle = "Engadget", .FeedBaseURI = "http://www.engadget.com/rss.xml"}, New Feed With {.FeedTitle = "Wired", .FeedBaseURI = "http://feeds.wired.com/wired/index"}))
        End Sub

        ''' <summary>
        ''' Downloads and stores the provided list of categories into the database.
        ''' </summary>
        ''' <param name="selection">An IList of categories selected by the user.</param>
        Public Sub LoadSelection(ByVal selection As IList)
            If Nothing IsNot selection Then
                For Each initCat As InitialCategory In selection
                    If Nothing IsNot initCat Then
                        ' Creating a new category with the provided InitialCategory's title
                        ' (Note: Casting it to a category yields a strange database error).
                        Dim cat As Category = New Category With {.CategoryTitle = initCat.CategoryTitle}

                        ' Add each Category to the database firstly so they will get their CategoryId's assigned.
                        App.DataBaseUtility.AddCategory(cat)

                        If Nothing IsNot initCat.Feeds Then
                            Dim feedList As List(Of Feed) = TryCast(initCat.Feeds, List(Of Feed))
                            If Nothing IsNot feedList Then
                                For Each feed In feedList
                                    ' Add each Initial Category's feeds to the database secondly so they will have their FeedId's assigned.
                                    App.DataBaseUtility.AddFeed(feed, cat)

                                Next feed
                            End If
                        Else
                            Throw New ArgumentNullException("The provided Initial Categories must contain valid Feeds.")
                        End If
                    Else
                        Throw New ArgumentException("One or more Initial Categories provided were null.")
                    End If
                Next initCat

                Dim f As List(Of Feed) = App.DataBaseUtility.GetAllFeeds
                LoadContent.DownloadFeeds(f)
            Else
                Throw New ArgumentNullException("Selection must not be null.", "selection")
            End If
        End Sub
    End Class
End Namespace