﻿' 
'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  
'

Imports System.Collections.ObjectModel
Imports System.Windows.Threading
Imports FeedCastLibrary
Imports FeedCast.Models

Namespace ViewModels
    ''' <summary>
    ''' ViewModel for the mainpage's what's new panorama item.
    ''' Add Articles here to populate that section of the UI.
    ''' </summary>
    Public Class MainPageWhatsNewViewModel
        Inherits ObservableCollection(Of Article)

        ' <summary>
        ' ViewModel for What's New panel, handles what articles are shown & what happens when refresh is tapped. 
        ' </summary>
        Public Sub New()
            Deployment.Current.Dispatcher.BeginInvoke(Sub()
                                                          Dim articleList As List(Of Article) = App.DataBaseUtility.WhatsNewCollection(20)
                                                          For Each a In articleList
                                                              Add(a)
                                                          Next a
                                                      End Sub)

            'Update!!
            If Not Settings.InitialLaunchSetting Then
                'LoadContent()
            End If
        End Sub

        ''' <summary>
        ''' Gets all the Feeds from the Database
        ''' </summary>
        Public Sub LoadContent(ByVal callback As Action)
            Dim loadContent As New ContentLoader(callback)
            AddHandler loadContent.LoadingFinished, AddressOf DisplayFeeds
            loadContent.DownloadFeeds(App.DataBaseUtility.GetAllFeeds)
        End Sub

        ''' <summary>
        ''' Saves Articles to the Database 
        ''' Displays the 20 most recent. 
        ''' </summary>
        ''' <param name="sender"></param>
        ''' <param name="e"></param>
        Public Sub DisplayFeeds(ByVal sender As Object, ByVal e As ContentLoader.LoadingFinishedEventArgs)
            'Pop on some on top, remove from the bottom
            'If empty then just pull the 20 most recent.
            ' In this scenario, the user removes all the feeds, adds some, and taps refresh. 
            Deployment.Current.Dispatcher.BeginInvoke(Sub()
                                                          If Me.Count > 0 Then
                                                              Dim newArticles As List(Of Article) = App.DataBaseUtility.UpdateWhatsNewCollection
                                                              For i = 0 To newArticles.Count - 1
                                                                  Insert(0, newArticles(i))
                                                                  If newArticles.Count >= 20 AndAlso i >= 20 Then
                                                                      RemoveAt(10)
                                                                  End If
                                                              Next i
                                                          Else
                                                              For Each a In App.DataBaseUtility.WhatsNewCollection(20)
                                                                  Add(a)
                                                              Next a
                                                          End If
                                                          TryCast(sender, ContentLoader)._callback()
                                                      End Sub)
        End Sub
    End Class
End Namespace