﻿' 
'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  
'
Imports System.Collections.ObjectModel
Imports System.Collections.Generic
Imports FeedCastLibrary

Namespace ViewModels
    Public Class NewFeedPageViewModel
        Inherits ObservableCollection(Of Article)

        ''' <summary>
        ''' The number of results to display.
        ''' </summary>
        Private _numOfResults As Integer

        ''' <summary>
        ''' Object to moderate threaded access.
        ''' </summary>
        Private ReadOnly _lockObject As Object

        ''' <summary>
        ''' The WebTools object used to download the articles of a feed.
        ''' </summary>
        Private Property _feedSearch As WebTools

        ''' <summary>
        ''' Returns whether FeedSearch is currently downloading results.
        ''' </summary>
        Public ReadOnly Property IsDownloading As Boolean
            Get
                If Nothing IsNot _feedSearch Then
                    Return _feedSearch.IsDownloading
                End If
                Return False
            End Get
        End Property

        ''' <summary>
        ''' Constructor
        ''' </summary>
        Public Sub New()
            _numOfResults = 10
            _lockObject = New Object
            _feedSearch = New WebTools(New SearchResultParser)
            Clear()
        End Sub

        Private Function GetSearchString(ByVal query As String) As String
            ' Format the search string.
            Dim search As String = "http://api.bing.com/rss.aspx?query=feed:" & query & "&source=web&web.count=" & _numOfResults.ToString & "&web.filetype=feed&market=en-us"
            Return search
        End Function

        ''' <summary>
        ''' Performs a search and adds the results to the observable collection.
        ''' </summary>
        ''' <param name="query">The query to search by</param>
        Public Sub GetResults(ByVal query As String, ByVal Callback As Action(Of Integer))
            ' Clear the page ViewModel.
            Clear()

            ' Get the search string and put it into a feed. 
            Dim feed As Feed = New Feed With {.FeedBaseURI = GetSearchString(query)}

            ' Lambda expression to add results to page VM after the download completes.
            ' _feedSearch is a WebTools object.
            AddHandler _feedSearch.AllDownloadsFinished, Sub(sender, e)
                                                             ' See if the search returned any results.
                                                             If e.Downloads.Count > 0 Then
                                                                 ' Add the search results to the page ViewModel.
                                                                 For Each result As Collection(Of Article) In e.Downloads.Values
                                                                     Dim reresult As Collection(Of Article) = result
                                                                     If Nothing IsNot result Then
                                                                         ' Add to the page VM.
                                                                         Deployment.Current.Dispatcher.BeginInvoke(Sub()
                                                                                                                       For Each a In reresult
                                                                                                                           SyncLock _lockObject
                                                                                                                               Add(a)
                                                                                                                           End SyncLock
                                                                                                                       Next a
                                                                                                                       Callback(Count)
                                                                                                                   End Sub)
                                                                     End If
                                                                 Next result
                                                             Else
                                                                 Deployment.Current.Dispatcher.BeginInvoke(Sub() Callback(0))
                                                             End If
                                                         End Sub

            ' Initiate the download (a Bing search).
            _feedSearch.Download(feed)
        End Sub

        ''' <summary>
        ''' Turns SearchResults into Feeds and adds them to the database.
        ''' </summary>
        ''' <param name="category">The category the feed(s) will be added to</param>
        ''' <param name="results">The collection of SearchResults</param>
        Public Sub ResultToFeed(ByVal category As Category, ByVal results As Collection(Of Article))
            Dim feedsAdded As New List(Of Feed)
            For Each result In results
                Dim feed As Feed = New Feed With {.FeedTitle = result.ArticleTitle, .FeedBaseURI = result.ArticleBaseURI}
                If App.DataBaseUtility.AddFeed(feed, category) Then
                    feedsAdded.Add(feed)
                End If
            Next result

            ' Load the articles for the new feeds.
            Dim loadContent As New ContentLoader
            loadContent.DownloadFeeds(feedsAdded)
            ' loadContent.LoadingFinished += MainPage._whatsNewArticles.DisplayFeeds
        End Sub
    End Class
End Namespace