﻿' 
'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  
'
Imports System.Windows.Media.Animation
Imports System.Windows.Navigation
Imports FeedCast.ViewModels
Imports FeedCastLibrary
Imports FeedCast.Resources

Namespace Views
    Partial Public Class ArticlePage
        Inherits PhoneApplicationPage

        ''' <summary>
        ''' How much time (in ms.) the title takes to expand when tapped.
        ''' </summary>
        Private Shared ReadOnly ExpansionAnimationDuration As Double = 200.0R

        ''' <summary>
        ''' 
        ''' </summary>
        Private _previousButton As ApplicationBarIconButton

        ''' <summary>
        ''' 
        ''' </summary>
        Private _nextButton As ApplicationBarIconButton

        ''' <summary>
        ''' 
        ''' </summary>
        Private _allArticles As List(Of Article)

        ''' <summary>
        ''' The article this page reflects.
        ''' </summary>
        Private _PageArticle As Article
        Public Property PageArticle As Article
            Get
                Return _PageArticle
            End Get
            Private Set(ByVal value As Article)
                _PageArticle = value
            End Set
        End Property

#Region "TitleMaxHeight Dependency Property"
        ''' <summary>
        ''' The default maximum amount of title content to show.
        ''' Defaults to 130.
        ''' </summary>
        Public Property TitleMaxHeight As Double
            Get
                Return CDbl(GetValue(TitleMaxHeightProperty))
            End Get
            Set(ByVal value As Double)
                SetValue(TitleMaxHeightProperty, value)
            End Set
        End Property

        ''' <summary>
        ''' The dependency property for the default maximum amount of title content to show.
        ''' Defaults to 130.
        ''' </summary>
        Public Shared ReadOnly TitleMaxHeightProperty As DependencyProperty = DependencyProperty.Register("MaxHeight", GetType(Double), GetType(ArticlePage), New PropertyMetadata(130.0R))
#End Region

        ''' <summary>
        ''' Whether the title is currently expanded.
        ''' </summary>
        Private _titleExpanded As Boolean

        Public Sub New()
            InitializeComponent()

            _titleExpanded = False
        End Sub

        ''' <summary>
        ''' Called when this page is navigated to; retrieve the article to show.
        ''' </summary>
        Protected Overrides Sub OnNavigatedTo(ByVal e As NavigationEventArgs)
            MyBase.OnNavigatedTo(e)

            ' Retrieve the article ID to retrive and populate the page with such article.
            Dim articleIDStr As String = Nothing
            Dim catIDStr As String = ""
            If NavigationContext.QueryString.TryGetValue("id", articleIDStr) Then
                ' Filter the article string.
                Dim articleSource As String = ""
                If articleIDStr.Contains("/") Then
                    ' Check to see if category id is contained. If so, user came from Category view.
                    If articleIDStr.Contains("/Category/") Then
                        articleSource = "category"
                        catIDStr = articleIDStr.Substring(articleIDStr.LastIndexOf("/") + 1)

                        ' Check to see if feed id is contained. If so, user came from Feed view.
                    ElseIf articleIDStr.Contains("/Feed") Then
                        articleSource = "feed"
                        ' Check to see if true is contained. If so, user came from Featured.
                    ElseIf articleIDStr.Contains("true") Then
                        articleSource = "featured"
                    End If

                    articleIDStr = articleIDStr.Remove(articleIDStr.IndexOf("/"))
                End If

                PageArticle = App.DataBaseUtility.QueryArticle(Integer.Parse(articleIDStr))

                If articleSource = "featured" Then
                    _allArticles = New List(Of Article)(Settings.FeaturedArticles)
                ElseIf articleSource = "feed" Then
                    _allArticles = New List(Of Article)(App.DataBaseUtility.GetFeedArticles(Convert.ToInt32(PageArticle.FeedID)))
                ElseIf articleSource = "category" Then
                    _allArticles = New List(Of Article)(App.DataBaseUtility.GetCategoryArticles(Convert.ToInt32(catIDStr)))
                Else
                    _allArticles = New List(Of Article)(App.DataBaseUtility.InitialWhatsNewCollection)
                End If

                Me.DataContext = PageArticle
            End If

            Dim appBar As New ApplicationBar

            Dim index As Integer = _allArticles.IndexOf(PageArticle)

            _previousButton = New ApplicationBarIconButton With {.IconUri = New Uri("/Icons/appbar.back.rest.png", UriKind.Relative), .Text = AppResources.AppBarButtonPreviousText, .IsEnabled = (index > 0)}

            AddHandler _previousButton.Click, AddressOf OnPreviousClick

            Dim browserButton As ApplicationBarIconButton = New ApplicationBarIconButton With {.IconUri = New Uri("/Icons/appbar.gotoaddressbar.rest.png", UriKind.Relative), .Text = AppResources.AppBarButtonOpenInBrowserText}

            AddHandler browserButton.Click, AddressOf OnBrowserClick

            _nextButton = New ApplicationBarIconButton With {.IconUri = New Uri("/Icons/appbar.next.rest.png", UriKind.Relative), .Text = AppResources.AppBarButtonNextText, .IsEnabled = (index < _allArticles.Count - 1)}

            AddHandler _nextButton.Click, AddressOf OnNextClick

            Dim shareMenuItem As New ApplicationBarMenuItem(AppResources.ShareText)

            AddHandler shareMenuItem.Click, AddressOf OnShareClick

            Dim delMenuItem As New ApplicationBarMenuItem(AppResources.ContextMenuRemoveText)

            AddHandler delMenuItem.Click, AddressOf OnDeleteClick

            Dim favMenuItem As New ApplicationBarMenuItem(AppResources.AddToFavoritesText)

            AddHandler favMenuItem.Click, AddressOf OnFavoriteClick

            appBar.Buttons.Add(_previousButton)
            appBar.Buttons.Add(browserButton)
            appBar.Buttons.Add(_nextButton)
            appBar.MenuItems.Add(shareMenuItem)
            appBar.MenuItems.Add(delMenuItem)
            appBar.MenuItems.Add(favMenuItem)

            Me.ApplicationBar = appBar
        End Sub

        ''' <summary>
        ''' User has clicked browser; launch the browser to this article's corresponding adress.
        ''' </summary>
        ''' <param name="sender"></param>
        ''' <param name="e"></param>
        Private Sub OnBrowserClick(ByVal sender As Object, ByVal e As EventArgs)
            If Nothing IsNot PageArticle Then
                ShareUtility.LaunchBrowser(PageArticle.ArticleBaseURI)
            End If
        End Sub

        ''' <summary>
        ''' User has clicked previous; retrieve the previous article to show the user.
        ''' </summary>
        Private Sub OnPreviousClick(ByVal sender As Object, ByVal e As EventArgs)
            If Nothing IsNot _allArticles AndAlso _allArticles.Count > 0 Then
                Dim index As Integer = _allArticles.IndexOf(PageArticle) - 1
                If index >= 0 Then
                    PageArticle = _allArticles(index)
                    If Not Convert.ToBoolean(PageArticle.Read) Then
                        PageArticle.Read = True
                        Dim feed As Feed = App.DataBaseUtility.QueryFeed(Convert.ToInt32(PageArticle.FeedID))
                        feed.UnreadCount -= 1
                        feed.ViewCount += 1
                        App.DataBaseUtility.SaveChangesToDB()
                    End If

                    Me.DataContext = PageArticle
                    _nextButton.IsEnabled = True
                    _previousButton.IsEnabled = (index > 0)
                End If
            End If
        End Sub

        ''' <summary>
        ''' User has clicked on next; retrieve the next article to show the user.
        ''' </summary>
        Private Sub OnNextClick(ByVal sender As Object, ByVal e As EventArgs)
            If Nothing IsNot _allArticles AndAlso _allArticles.Count > 0 Then
                Dim index As Integer = _allArticles.IndexOf(PageArticle) + 1
                If index < _allArticles.Count Then
                    PageArticle = _allArticles(index)
                    If Not Convert.ToBoolean(PageArticle.Read) Then
                        PageArticle.Read = True
                        Dim feed As Feed = App.DataBaseUtility.QueryFeed(Convert.ToInt32(PageArticle.FeedID))
                        feed.UnreadCount -= 1
                        feed.ViewCount += 1
                        App.DataBaseUtility.SaveChangesToDB()
                    End If

                    Me.DataContext = PageArticle
                    _previousButton.IsEnabled = True
                    _nextButton.IsEnabled = (index < _allArticles.Count - 1)
                End If
            End If
        End Sub

        ''' <summary>
        ''' User has clicked on share; navigate to share options.
        ''' </summary>
        Private Sub OnShareClick(ByVal sender As Object, ByVal e As EventArgs)
            If Nothing IsNot PageArticle Then
                App.DataBaseUtility.QueryFeed(Convert.ToInt32(PageArticle.FeedID)).SharedCount += 1
                NavigationService.Navigate(New Uri("/Share/" & PageArticle.ArticleBaseURI, UriKind.Relative))
            End If
        End Sub

        ''' <summary>
        ''' User has clicked on delete; Article is deleted from the Database and UI.
        ''' </summary>
        Private Sub OnDeleteClick(ByVal sender As Object, ByVal e As EventArgs)
            ' Determine the deleted article/index
            Dim deletedIndex As Integer = _allArticles.IndexOf(PageArticle)
            Dim deletedArticle As Article = PageArticle

            If Not Convert.ToBoolean(deletedArticle.Read) Then
                deletedArticle.Read = True
                Dim feed As Feed = App.DataBaseUtility.QueryFeed(Convert.ToInt32(deletedArticle.FeedID))
                feed.UnreadCount -= 1
                App.DataBaseUtility.SaveChangesToDB()
            End If

            ' Remove it from the database/ui
            App.DataBaseUtility.DeleteArticle(deletedArticle)
            _allArticles.Remove(deletedArticle)
            MainPage._featuredArticles.RedoFeatured()

            If NavigationService.CanGoBack Then
                NavigationService.GoBack()
            End If

            '' Determine where to move.
            'Dim movedArticle As Article = _allArticles(deletedIndex)

            '' Add the moved article to the ui.
            'PageArticle = movedArticle
            'PageArticle.Read = True
            'App.DataBaseUtility.SaveChangesToDB
            'Me.DataContext = PageArticle

            '' Determine trhe status of the buttons.
            '_nextButton.IsEnabled = (deletedIndex < _allArticles.Count - 1)
            '_previousButton.IsEnabled = (deletedIndex > 0)
        End Sub

        ''' <summary>
        ''' User has clicked on Add to Favorites; article is added to favorites.
        ''' </summary>
        Private Sub OnFavoriteClick(ByVal sender As Object, ByVal e As EventArgs)
            PageArticle.Favorite = True
            App.DataBaseUtility.AddCat_Feed(Convert.ToInt32(PageArticle.FeedID), 1)
            App.DataBaseUtility.SaveChangesToDB()
        End Sub

        ''' <summary>
        ''' Animate the title to show the rest of it's content.
        ''' </summary>
        Private Sub OnTitleTap(ByVal sender As Object, ByVal e As EventArgs)
            Dim title As TextBlock = TryCast(sender, TextBlock)
            If Nothing IsNot title AndAlso (_titleExpanded OrElse (title.ActualHeight > title.MaxHeight)) Then
                Dim duration As New Duration(TimeSpan.FromMilliseconds(ExpansionAnimationDuration))

                Dim heightAnim As New DoubleAnimation With {.To = If(_titleExpanded, TitleMaxHeight, title.ActualHeight), .Duration = duration}

                Dim sb As New Storyboard With {.Duration = duration}
                Storyboard.SetTargetProperty(heightAnim, New PropertyPath(TextBlock.MaxHeightProperty))
                Storyboard.SetTarget(heightAnim, title)
                sb.Children.Add(heightAnim)
                sb.Begin()

                _titleExpanded = Not _titleExpanded
            End If
        End Sub
    End Class
End Namespace