﻿' 
'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  
'

Imports System.Collections.ObjectModel
Imports System.Windows.Navigation
Imports FeedCastLibrary
Imports FeedCast.Resources

Namespace Views
    ''' <summary>
    ''' Page that shows the user all articles within a selected category.
    ''' </summary>
    Partial Public Class CategoryPage
        Inherits PhoneApplicationPage

        ''' <summary>
        ''' 
        ''' </summary>
        Private _previousButton As ApplicationBarIconButton

        ''' <summary>
        ''' 
        ''' </summary>
        Private _nextButton As ApplicationBarIconButton

        ''' <summary>
        ''' 
        ''' </summary>
        Private _allCategories As List(Of Category)

        ''' <summary>
        ''' 
        ''' </summary>
        Private _allCategoryArticles As List(Of Article)

        ''' <summary>
        ''' The category the user selected.
        ''' </summary>
        Public Property SelectedCategory As Category
            Get
                Return CType(GetValue(SelectedCategoryProperty), Category)
            End Get
            Set(ByVal value As Category)
                SetValue(SelectedCategoryProperty, value)
            End Set
        End Property

        ' Using a DependencyProperty as the backing store for SelectedCategory.  This enables animation, styling, binding, etc...
        Public Shared ReadOnly SelectedCategoryProperty As DependencyProperty = DependencyProperty.Register("SelectedCategory", GetType(Category), GetType(CategoryPage), New PropertyMetadata(Nothing))

        ''' <summary>
        ''' All the articles belonging to the category selected by the user.
        ''' </summary>
        Private _categoryArticles As ObservableCollection(Of Article)
        Public Property CategoryArticles As ObservableCollection(Of Article)
            Get
                Return _categoryArticles
            End Get
            Private Set(ByVal value As ObservableCollection(Of Article))
                _categoryArticles = value
            End Set
        End Property

        Public Sub New()
            InitializeComponent()
        End Sub

        ''' <summary>
        ''' Retrieve the category clicked by the user and it's corresponding articles.
        ''' </summary>
        Protected Overrides Sub OnNavigatedTo(ByVal e As NavigationEventArgs)
            MyBase.OnNavigatedTo(e)

            Dim idStr As String = Nothing
            If NavigationContext.QueryString.TryGetValue("id", idStr) Then
                SelectedCategory = App.DataBaseUtility.QueryCategory(Convert.ToInt32(idStr))
                _allCategoryArticles = App.DataBaseUtility.GetCategoryArticles(Convert.ToInt32(idStr))
                Dim firstSetOfArticles As New List(Of Article)(_allCategoryArticles)
                If firstSetOfArticles.Count > 10 Then
                    firstSetOfArticles.RemoveRange(10, firstSetOfArticles.Count - 10)
                End If
                CategoryArticles = New ObservableCollection(Of Article)(firstSetOfArticles)
                DataContext = CategoryArticles
            End If

            _allCategories = App.DataBaseUtility.GetAllCategories
            _allCategories.Sort()

            Dim appBar As New ApplicationBar

            Dim index As Integer = _allCategories.IndexOf(SelectedCategory)

            _previousButton = New ApplicationBarIconButton With {.IconUri = New Uri("/Icons/appbar.back.rest.png", UriKind.Relative), .Text = AppResources.AppBarButtonPreviousText, .IsEnabled = (index > 0)}

            AddHandler _previousButton.Click, AddressOf OnPreviousClick

            Dim addMenu As New ApplicationBarMenuItem(AppResources.AppBarMenuItemAddText)

            AddHandler addMenu.Click, AddressOf OnAddMenuClick

            _nextButton = New ApplicationBarIconButton With {.IconUri = New Uri("/Icons/appbar.next.rest.png", UriKind.Relative), .Text = AppResources.AppBarButtonNextText, .IsEnabled = (index < _allCategories.Count - 1)}

            AddHandler _nextButton.Click, AddressOf OnNextClick

            appBar.Buttons.Add(_previousButton)
            appBar.Buttons.Add(_nextButton)
            appBar.MenuItems.Add(addMenu)

            Me.ApplicationBar = appBar
        End Sub

        ''' <summary>
        ''' User has tapped on an article, navigate to article page and show the tapped article.
        ''' </summary>
        Private Sub OnArticleTap(ByVal sender As Object, ByVal e As EventArgs)
            Dim container As FrameworkElement = TryCast(sender, FrameworkElement)
            If Nothing IsNot container AndAlso Nothing IsNot container.Tag Then
                Dim tappedArticle As Article = TryCast(container.Tag, Article)
                If Nothing IsNot tappedArticle Then
                    If Not Convert.ToBoolean(tappedArticle.Read) Then
                        tappedArticle.Read = True
                        App.DataBaseUtility.QueryFeed(Convert.ToInt32(tappedArticle.FeedID)).UnreadCount -= 1
                        App.DataBaseUtility.QueryFeed(Convert.ToInt32(tappedArticle.FeedID)).ViewCount += 1
                        App.DataBaseUtility.SaveChangesToDB()
                    End If
                    NavigationService.Navigate(New Uri("/Article/" & tappedArticle.ArticleID & "/Category/" & SelectedCategory.CategoryID, UriKind.Relative))
                End If
            End If
        End Sub

        ''' <summary>
        ''' Remove the selected article.
        ''' </summary>
        Private Sub OnRemoveClick(ByVal sender As Object, ByVal e As EventArgs)
            Dim menu As MenuItem = TryCast(sender, MenuItem)
            If Nothing IsNot sender Then
                Dim removed As Article = TryCast(menu.Tag, Article)
                If Nothing IsNot removed Then

                    If Not Convert.ToBoolean(removed.Read) Then
                        Dim feed As Feed = App.DataBaseUtility.QueryFeed(Convert.ToInt32(removed.FeedID))
                        feed.UnreadCount -= 1
                        App.DataBaseUtility.SaveChangesToDB()
                    End If

                    CategoryArticles.Remove(removed)
                    ' TODO check if article is removed in the database.
                    App.DataBaseUtility.DeleteArticle(removed)
                End If
            End If
        End Sub

        ''' <summary>
        ''' Navigate to the previous category.
        ''' </summary>
        Private Sub OnPreviousClick(ByVal sender As Object, ByVal e As EventArgs)
            If Nothing IsNot _allCategories AndAlso _allCategories.Count > 0 Then
                Dim prevCatIndex As Integer = _allCategories.IndexOf(SelectedCategory) - 1
                If prevCatIndex >= 0 Then
                    Dim previousCategory As Category = _allCategories(prevCatIndex)
                    Dim categoryArticles As List(Of Article) = App.DataBaseUtility.GetCategoryArticles(previousCategory.CategoryID)
                    If Nothing IsNot categoryArticles Then
                        Me.SelectedCategory = previousCategory
                        Me.DataContext = categoryArticles
                        _nextButton.IsEnabled = True
                        _previousButton.IsEnabled = (prevCatIndex > 0)
                    End If
                End If
            End If
        End Sub

        ''' <summary>
        ''' Navigate to the next category.
        ''' </summary>
        Private Sub OnNextClick(ByVal sender As Object, ByVal e As EventArgs)
            If Nothing IsNot _allCategories AndAlso _allCategories.Count > 0 Then
                Dim nextCatIndex As Integer = _allCategories.IndexOf(SelectedCategory) + 1
                If nextCatIndex < _allCategories.Count Then
                    Dim previousCategory As Category = _allCategories(nextCatIndex)
                    Dim categoryArticles As List(Of Article) = App.DataBaseUtility.GetCategoryArticles(previousCategory.CategoryID)
                    If Nothing IsNot categoryArticles Then
                        Me.SelectedCategory = previousCategory
                        Me.DataContext = categoryArticles
                        _previousButton.IsEnabled = True
                        _nextButton.IsEnabled = (nextCatIndex < _allCategories.Count - 1)
                    End If
                End If
            End If
        End Sub

        ''' <summary>
        ''' Handler called when the user presses add in the application bar menu.
        ''' </summary>
        Public Sub OnAddMenuClick(ByVal sender As Object, ByVal e As EventArgs)
            NavigationService.Navigate(New Uri("/NewFeed/" & SelectedCategory.CategoryID, UriKind.Relative))
        End Sub

        Private Sub OnBottomReached(ByVal sender As Object, ByVal e As EventArgs)
            If CategoryArticles.Count <> 0 AndAlso _allCategoryArticles.Count > CategoryArticles.Count Then
                Dim nextSetOfArticles As New List(Of Article)(_allCategoryArticles)
                If Nothing IsNot nextSetOfArticles Then
                    Dim [set] As Array
                    If CategoryArticles.Count + 10 < nextSetOfArticles.Count Then
                        [set] = nextSetOfArticles.GetRange(CategoryArticles.Count, 10).ToArray
                    Else
                        [set] = nextSetOfArticles.GetRange(CategoryArticles.Count, nextSetOfArticles.Count - CategoryArticles.Count).ToArray
                    End If
                    For Each a In [set]
                        CategoryArticles.Add(a)
                    Next a
                End If
            End If
        End Sub
    End Class
End Namespace