﻿' 
'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  
'

Imports System.Windows.Navigation
Imports FeedCastLibrary
Imports FeedCast.Resources
Imports System.Collections.ObjectModel

Namespace Views
    ''' <summary>
    ''' Page that shows the user all articles within a selected feed.
    ''' </summary>
    Partial Public Class FeedPage
        Inherits PhoneApplicationPage

        ''' <summary>
        ''' 
        ''' </summary>
        Private _previousButton As ApplicationBarIconButton

        ''' <summary>
        ''' 
        ''' </summary>
        Private _nextButton As ApplicationBarIconButton

        ''' <summary>
        ''' 
        ''' </summary>
        Private _allFeeds As List(Of Feed)

        ''' <summary>
        ''' 
        ''' </summary>
        Private _allFeedsArticles As List(Of Article)

        ''' <summary>
        ''' The feed selected by the user whose articles to display.
        ''' </summary>
        Public Property SelectedFeed As Feed
            Get
                Return CType(GetValue(SelectedFeedProperty), Feed)
            End Get
            Set(ByVal value As Feed)
                SetValue(SelectedFeedProperty, value)
            End Set
        End Property

        ' Using a DependencyProperty as the backing store for SelectedFeed.  This enables animation, styling, binding, etc...
        Public Shared ReadOnly SelectedFeedProperty As DependencyProperty = DependencyProperty.Register("SelectedFeed", GetType(Feed), GetType(FeedPage), New PropertyMetadata(Nothing))

        ''' <summary>
        ''' The articles belonging to the feed selected by the user.
        ''' </summary>
        Private _feedArticles As ObservableCollection(Of Article)
        Public Property FeedArticles As ObservableCollection(Of Article)
            Get
                Return _feedArticles
            End Get
            Private Set(ByVal value As ObservableCollection(Of Article))
                _feedArticles = value
            End Set
        End Property

        Public Sub New()
            InitializeComponent()
        End Sub

        ''' <summary>
        ''' Retrieve the feed selected by the user and all it's corresponding articles.
        ''' </summary>
        Protected Overrides Sub OnNavigatedTo(ByVal e As NavigationEventArgs)
            MyBase.OnNavigatedTo(e)

            Dim idStr As String = Nothing
            If NavigationContext.QueryString.TryGetValue("id", idStr) Then
                SelectedFeed = App.DataBaseUtility.QueryFeed(Convert.ToInt32(idStr))
                _allFeedsArticles = App.DataBaseUtility.GetFeedArticles(Convert.ToInt32(idStr))
                Dim firstSetOfArticles As New List(Of Article)(_allFeedsArticles)
                If firstSetOfArticles.Count > 10 Then
                    firstSetOfArticles.RemoveRange(10, firstSetOfArticles.Count - 10)
                End If

                FeedArticles = New ObservableCollection(Of Article)(firstSetOfArticles)
                DataContext = FeedArticles
            End If

            _allFeeds = App.DataBaseUtility.GetAllFeeds
            _allFeeds.Sort()
            Dim appBar As ApplicationBar = New ApplicationBar With {.IsMenuEnabled = False}

            Dim index As Integer = _allFeeds.IndexOf(SelectedFeed)

            _previousButton = New ApplicationBarIconButton With {.IconUri = New Uri("/Icons/appbar.back.rest.png", UriKind.Relative), .Text = AppResources.AppBarButtonPreviousText, .IsEnabled = (index > 0)}

            AddHandler _previousButton.Click, AddressOf OnPreviousClick

            _nextButton = New ApplicationBarIconButton With {.IconUri = New Uri("/Icons/appbar.next.rest.png", UriKind.Relative), .Text = AppResources.AppBarButtonNextText, .IsEnabled = (index < _allFeeds.Count - 1)}

            AddHandler _nextButton.Click, AddressOf OnNextClick

            appBar.Buttons.Add(_previousButton)
            appBar.Buttons.Add(_nextButton)

            Me.ApplicationBar = appBar
        End Sub

        ''' <summary>
        ''' User has tapped on an article, navigate to article page and show the tapped article.
        ''' </summary>
        Private Sub OnArticleTap(ByVal sender As Object, ByVal e As EventArgs)
            Dim container As FrameworkElement = TryCast(sender, FrameworkElement)
            If Nothing IsNot container AndAlso Nothing IsNot container.Tag Then
                Dim tappedArticle As Article = TryCast(container.Tag, Article)
                If Nothing IsNot tappedArticle Then
                    If Not Convert.ToBoolean(tappedArticle.Read) Then
                        tappedArticle.Read = True
                        App.DataBaseUtility.QueryFeed(Convert.ToInt32(tappedArticle.FeedID)).UnreadCount -= 1
                        App.DataBaseUtility.QueryFeed(Convert.ToInt32(tappedArticle.FeedID)).ViewCount += 1
                        App.DataBaseUtility.SaveChangesToDB()
                    End If
                    NavigationService.Navigate(New Uri("/Article/" & tappedArticle.ArticleID & "/Feed", UriKind.Relative))
                End If
            End If
        End Sub

        ''' <summary>
        ''' Remove the specified article.
        ''' </summary>
        Private Sub OnRemoveClick(ByVal sender As Object, ByVal e As EventArgs)
            Dim menu As MenuItem = TryCast(sender, MenuItem)
            If Nothing IsNot sender Then
                Dim removed As Article = TryCast(menu.Tag, Article)
                If Nothing IsNot removed Then
                    FeedArticles.Remove(removed)
                    ' TODO check if article is removed in the database.
                    App.DataBaseUtility.DeleteArticle(removed)
                End If
            End If
        End Sub

        ''' <summary>
        ''' Display the previous feed.
        ''' </summary>
        Private Sub OnPreviousClick(ByVal sender As Object, ByVal e As EventArgs)
            If Nothing IsNot _allFeeds AndAlso _allFeeds.Count > 0 Then
                Dim prevFeedIndex As Integer = _allFeeds.IndexOf(SelectedFeed) - 1
                If prevFeedIndex >= 0 Then
                    Dim previousFeed As Feed = _allFeeds(prevFeedIndex)
                    Dim feedArticles As List(Of Article) = App.DataBaseUtility.GetFeedArticles(previousFeed.FeedID)
                    If Nothing IsNot _feedArticles Then
                        Me.SelectedFeed = previousFeed
                        Me.DataContext = _feedArticles
                        _nextButton.IsEnabled = True
                        _previousButton.IsEnabled = (prevFeedIndex > 0)
                    End If
                End If
            End If
        End Sub

        ''' <summary>
        ''' Display the next feed.
        ''' </summary>
        Private Sub OnNextClick(ByVal sender As Object, ByVal e As EventArgs)
            If Nothing IsNot _allFeeds AndAlso _allFeeds.Count > 0 Then
                Dim nextFeedIndex As Integer = _allFeeds.IndexOf(SelectedFeed) + 1
                If nextFeedIndex < _allFeeds.Count Then
                    Dim previousFeed As Feed = _allFeeds(nextFeedIndex)
                    Dim feedArticles As List(Of Article) = App.DataBaseUtility.GetFeedArticles(previousFeed.FeedID)
                    If Nothing IsNot _feedArticles Then
                        Me.SelectedFeed = previousFeed
                        Me.DataContext = _feedArticles
                        _previousButton.IsEnabled = True
                        _nextButton.IsEnabled = (nextFeedIndex < _allFeeds.Count - 1)
                    End If
                End If
            End If
        End Sub

        Private Sub OnBottomReached(ByVal sender As Object, ByVal e As EventArgs)
            If FeedArticles.Count <> 0 AndAlso _allFeedsArticles.Count > FeedArticles.Count Then
                Dim nextSetOfArticles As New List(Of Article)(_allFeedsArticles)
                If Nothing IsNot nextSetOfArticles Then
                    Dim [set] As Array
                    If FeedArticles.Count + 10 < nextSetOfArticles.Count Then
                        [set] = nextSetOfArticles.GetRange(FeedArticles.Count, 10).ToArray
                    Else
                        [set] = nextSetOfArticles.GetRange(FeedArticles.Count, nextSetOfArticles.Count - FeedArticles.Count).ToArray
                    End If

                    For Each a In [set]
                        FeedArticles.Add(a)
                    Next a
                End If
            End If
        End Sub
    End Class
End Namespace