﻿' 
'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  
'

Imports System.Collections
Imports System.Windows.Controls.Primitives
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Media.Imaging
Imports System.Windows.Threading
Imports FeedCast.ViewModels
Imports FeedCastLibrary
Imports Microsoft.Phone.Controls
Imports FeedCast.Resources

Namespace Views
    ''' <summary>
    ''' The initial application page, that only shows once, where the user
    ''' decides what categories he would initially like to populate their reader with.
    ''' </summary>
    Partial Public Class LaunchPage
        Inherits PhoneApplicationPage

        ''' <summary>
        ''' Collection of initial categories displayed to the user.
        ''' </summary>
        Private _initialCategories As LaunchPageViewModel

#Region "Fade Effect"
        ''' <summary>
        ''' 
        ''' </summary>
        Private _indicators As Panel

        ''' <summary>
        ''' 
        ''' </summary>
        Private _errorText As TextBlock

        ''' <summary>
        ''' 
        ''' </summary>
        Private _loadingText As TextBlock

        ''' <summary>
        ''' 
        ''' </summary>
        Private _progressBar As PerformanceProgressBar

        ''' <summary>
        ''' Storyboard for tresizing the background fading effect.
        ''' </summary>
        Private _backgroundResizeStoryboard As Storyboard

        ''' <summary>
        ''' Popup used to display the Fade Effect.
        ''' </summary>
        Private _popup As Popup

        ''' <summary>
        ''' Overlay containing layers for the Fade Effect.
        ''' </summary>
        Private _overlay As Panel
#End Region

        ''' <summary>
        ''' Constructor called when the page is initialized.
        ''' </summary>
        Public Sub New()
            InitializeComponent()

            ' Load all initial categories into memory.
            _initialCategories = New LaunchPageViewModel

            ' Hook into the DownloadsFinished event for all initial categories.
            AddHandler _initialCategories.LoadContent.LoadingFinished, AddressOf OnAllDownloadsFinished
        End Sub

#Region "Navigation Overrides"
        ''' <summary>
        ''' Called when the page is first navigated to.
        ''' </summary>
        Protected Overrides Sub OnNavigatedTo(ByVal e As NavigationEventArgs)
            MyBase.OnNavigatedTo(e)

            ' Set the datacontext only once when the page is navigated to.
            CategorySelection.DataContext = _initialCategories
        End Sub

        ''' <summary>
        ''' Called when the page is no longer the active page in the root frame. 
        ''' </summary>
        ''' <param name="e">The Navigation Event Arguments provided.</param>
        Protected Overrides Sub OnNavigatedFrom(ByVal e As NavigationEventArgs)
            MyBase.OnNavigatedFrom(e)

            ' Remove back entry to prevent the user from accessing this page again.
            If NavigationService.CanGoBack Then
                NavigationService.RemoveBackEntry()
            End If
        End Sub
#End Region

        ''' <summary>
        ''' Event handler called when the user cliks the Finished button.
        ''' </summary>
        ''' <param name="sender">The object that fired this event.</param>
        ''' <param name="e">The Event Arguments associated with this event.</param>
        Private Sub OnFinishedClicked(ByVal sender As Object, ByVal e As EventArgs)
            ' Begin the fade navigation to indicate loading.
            OpenPopup()

            ' Retrieve the InitialCategories selected by the user.
            Dim selectedItems As IList = TryCast(CategorySelection.SelectedItems, IList)
            If selectedItems.Count > 0 Then
                ' Begin loading the Initial Categories selected by the user.
                Try
                    _initialCategories.LoadSelection(selectedItems)
                Catch e1 As WebException
                    _errorText.Visibility = Visibility.Visible

                    Dim dt As New DispatcherTimer
                    dt.Interval = TimeSpan.FromSeconds(3.0R)
                    AddHandler dt.Tick, Sub(s, a)
                                            ClosePopup()
                                            dt.Stop()
                                        End Sub
                    dt.Start()
                End Try
            Else
                OnAllDownloadsFinished(sender, Nothing)
            End If
        End Sub

        ''' <summary>
        ''' Event handler called when all downloads have successfully finished.
        ''' </summary>
        ''' <param name="sender">The sender of this event handler.</param>
        ''' <param name="e">The EventArgs for this handler.</param>
        Private Sub OnAllDownloadsFinished(ByVal sender As Object, ByVal e As ContentLoader.LoadingFinishedEventArgs)
            Dispatcher.BeginInvoke(Sub()
                                       NavigationService.Navigate(New Uri("/Main", UriKind.Relative))
                                       ClosePopup()
                                   End Sub)
        End Sub

        ''' <summary>
        ''' Opens the ProgressBar Popup.
        ''' </summary>
        Private Sub OpenPopup()
            ' Grab the frame for measurements.
            Dim rootVisual As PhoneApplicationFrame = TryCast(Application.Current.RootVisual, PhoneApplicationFrame)

            ' Panel that contains overlay layers
            _overlay = New Canvas With {.Background = New SolidColorBrush(Colors.Transparent)}

            Dim width As Double = rootVisual.ActualWidth
            Dim height As Double = rootVisual.ActualHeight

            ' Create a layer for the background brush (Prevents edge seeping).
            Dim backgroundLayer As UIElement = New Rectangle With {.Width = width, .Height = height, .Fill = CType(Application.Current.Resources("PhoneBackgroundBrush"), Brush), .CacheMode = New BitmapCache}
            _overlay.Children.Insert(0, backgroundLayer)

            ' Create a layer for the page content
            Dim writeableBitmap As New WriteableBitmap(CInt(width), CInt(height))
            writeableBitmap.Render(rootVisual, Nothing)
            writeableBitmap.Invalidate()
            Dim scaleTransform As ScaleTransform = New ScaleTransform With {.CenterX = width / 2, .CenterY = height / 2}
            Dim contentLayer As UIElement = New Image With {.Source = writeableBitmap, .RenderTransform = scaleTransform, .CacheMode = New BitmapCache}
            _overlay.Children.Insert(1, contentLayer)

            ' Create a layer for the background brush
            Dim backgroundFadeLayer As UIElement = New Rectangle With {.Width = width, .Height = height, .Fill = CType(Application.Current.Resources("PhoneBackgroundBrush"), Brush), .Opacity = 0, .CacheMode = New BitmapCache}
            _overlay.Children.Insert(2, backgroundFadeLayer)

            ' Prepare for scale animation
            Dim [from] As Double = 1
            Dim [to] As Double = 0.94
            Dim timespan As TimeSpan = timespan.FromSeconds(0.42)
            Dim easingFunction As IEasingFunction = New ExponentialEase With {.EasingMode = EasingMode.EaseInOut}
            _backgroundResizeStoryboard = New Storyboard

            ' Create an animation for the X scale
            Dim animationX As DoubleAnimation = New DoubleAnimation With {.From = [from], .To = [to], .Duration = timespan, .EasingFunction = easingFunction}
            Storyboard.SetTarget(animationX, scaleTransform)
            Storyboard.SetTargetProperty(animationX, New PropertyPath(scaleTransform.ScaleXProperty))
            _backgroundResizeStoryboard.Children.Add(animationX)

            ' Create an animation for the Y scale
            Dim animationY As DoubleAnimation = New DoubleAnimation With {.From = [from], .To = [to], .Duration = timespan, .EasingFunction = easingFunction}
            Storyboard.SetTarget(animationY, scaleTransform)
            Storyboard.SetTargetProperty(animationY, New PropertyPath(scaleTransform.ScaleYProperty))
            _backgroundResizeStoryboard.Children.Add(animationY)

            Dim animationFade As DoubleAnimation = New DoubleAnimation With {.From = 0, .To = 0.8, .Duration = timespan, .EasingFunction = easingFunction}
            Storyboard.SetTarget(animationFade, backgroundFadeLayer)
            Storyboard.SetTargetProperty(animationFade, New PropertyPath(Rectangle.OpacityProperty))
            _backgroundResizeStoryboard.Children.Add(animationFade)

            _indicators = New StackPanel

            _progressBar = New PerformanceProgressBar With {.Width = width}

            _indicators.Children.Add(_progressBar)

            _loadingText = New TextBlock With {.Text = AppResources.InitialLoadingText, .Style = CType(Application.Current.Resources("PhoneTextTitle3Style"), Style), .TextAlignment = TextAlignment.Center, .Width = width}

            _indicators.Children.Add(_loadingText)

            _errorText = New TextBlock With {.Text = AppResources.InitialLoadingFailedText, .Style = CType(Application.Current.Resources("PhoneTextAccentStyle"), Style), .TextAlignment = TextAlignment.Center, .Width = width, .Visibility = Visibility.Collapsed}

            _indicators.Children.Add(_errorText)

            _overlay.Children.Add(_indicators)

            Canvas.SetLeft(_indicators, 0)
            Canvas.SetTop(_indicators, ((height / 2) - (_indicators.ActualHeight / 2)))

            _popup = New Popup With {.Child = _overlay}

            AddHandler _popup.Opened, Sub(s, e)
                                          _progressBar.IsIndeterminate = True
                                          _backgroundResizeStoryboard.Begin()
                                      End Sub

            AddHandler _popup.Closed, Sub(s, e)
                                          _progressBar.IsIndeterminate = False
                                          _progressBar.Visibility = Visibility.Collapsed

                                          _loadingText.Visibility = Visibility.Collapsed
                                      End Sub

            _popup.IsOpen = True
        End Sub

        ''' <summary>
        ''' Closes the Popup.
        ''' </summary>
        Private Sub ClosePopup()
            _indicators.Visibility = Visibility.Collapsed
            If Nothing IsNot _backgroundResizeStoryboard Then
                ' Swap all the From/To values to reverse the animation
                For Each animation As DoubleAnimation In _backgroundResizeStoryboard.Children
                    Dim temp As Double = animation.From.Value
                    animation.From = animation.To
                    animation.To = temp
                Next animation

                ' Capture member variables for delegate closure
                Dim popup As Popup = _popup
                Dim overlay As Panel = _overlay
                AddHandler _backgroundResizeStoryboard.Completed, Sub()
                                                                      ' Clear/close popup and overlay
                                                                      If Nothing IsNot popup Then
                                                                          popup.IsOpen = False
                                                                          popup.Child = Nothing
                                                                      End If
                                                                      If Nothing IsNot overlay Then
                                                                          overlay.Children.Clear()
                                                                      End If
                                                                  End Sub

                ' Begin the reverse animation
                _backgroundResizeStoryboard.Begin()

                ' Reset member variables
                _backgroundResizeStoryboard = Nothing
                _popup = Nothing
                _overlay = Nothing
            Else
                If Nothing IsNot _popup Then
                    _popup.IsOpen = False
                    _popup.Child = Nothing
                    _popup = Nothing
                End If
                If Nothing IsNot _overlay Then
                    _overlay.Children.Clear()
                    _overlay = Nothing
                End If
            End If
        End Sub
    End Class
End Namespace