﻿' 
'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  
'

Imports System.Windows.Navigation
Imports FeedCast.ViewModels
Imports FeedCastLibrary
Imports FeedCast.Resources
Imports System.Windows.Data
Imports FeedCast.Models

''' <summary>
''' The mainpage of the application. Allows the user to view recent articles,
''' add and view new feeds, categories, and offers "featured" suggestions.
''' </summary>
Partial Public Class MainPage
    Inherits PhoneApplicationPage

    ''' <summary>
    ''' Categories in the "All" PanoramaItem.
    ''' </summary>
    Public _allCategories As MainPageAllCategoriesViewModel

    ''' <summary>
    ''' Feeds in the "All" PanoramaItem.
    ''' </summary>
    Public _allFeeds As MainPageAllFeedsViewModel

    ''' <summary>
    ''' Articles in the "What's New" PanoramaItem.
    ''' </summary>
    Public _whatsNewArticles As MainPageWhatsNewViewModel

    ''' <summary>
    ''' Articles in the Featured Panorama Item
    ''' </summary>
    Public Shared _featuredArticles As MainPageFeaturedViewModel


    Public Sub New()
        InitializeComponent()
    End Sub

    Protected Overrides Sub OnNavigatedTo(ByVal e As NavigationEventArgs)
        MyBase.OnNavigatedTo(e)

        If Settings.InitialLaunchSetting Then
            App.Favorites = New Category With {.CategoryTitle = AppResources.FavoritesCategoryTitleText}
            App.DataBaseUtility.AddCategory(App.Favorites)
            App.DataBaseUtility.SaveChangesToDB()

            ' App has initialized for the first time.
            Settings.InitialLaunchSetting = False
        End If

        If e.NavigationMode = NavigationMode.New Then
            Dim appBar As ApplicationBar = New ApplicationBar With {.Mode = ApplicationBarMode.Minimized}

            Dim refreshMenu As New ApplicationBarMenuItem(AppResources.MainPageAppBarRefreshText)

            AddHandler refreshMenu.Click, AddressOf OnRefreshClick

            appBar.MenuItems.Add(refreshMenu)

            Me.ApplicationBar = appBar
        End If

        _allCategories = New MainPageAllCategoriesViewModel

        _allFeeds = New MainPageAllFeedsViewModel

        _whatsNewArticles = New MainPageWhatsNewViewModel

        _featuredArticles = New MainPageFeaturedViewModel


        ' Loading categories in the database to the "All" panoramaitem's categories listbox.
        AllCategoryItems.DataContext = _allCategories

        ' Loading feeds in the database to the "All" panoramaitem's feeds longlistselector.
        AllFeedItems.DataContext = _allFeeds

        ' Loading articles in the database to the "What's new" panoramaitem's article listbox.
        WhatsNewArticleItems.DataContext = _whatsNewArticles


        ' Loading featured tiles
        FeaturedArticleItems.DataContext = New MainPageFeaturedViewModel

        AddHandler HubTile1.Tap, AddressOf OnHubTileTapped
        AddHandler HubTile2.Tap, AddressOf OnHubTileTapped
        AddHandler HubTile3.Tap, AddressOf OnHubTileTapped
        AddHandler HubTile4.Tap, AddressOf OnHubTileTapped
        AddHandler HubTile5.Tap, AddressOf OnHubTileTapped
        AddHandler HubTile6.Tap, AddressOf OnHubTileTapped

        ' Loading settings
        SettingsPanel.DataContext = App.ApplicationSettings
    End Sub

    ''' <summary>
    ''' Called when navigating away from this page.
    ''' </summary>
    ''' <param name="e"></param>
    Protected Overrides Sub OnNavigatingFrom(ByVal e As NavigatingCancelEventArgs)
        MyBase.OnNavigatingFrom(e)

        ' Disable the loading bar.
        If LoadingProgressBar.IsIndeterminate Then
            LoadingProgressBar.IsIndeterminate = False
        End If
    End Sub

    ''' <summary>
    ''' Called when the user navigates between panoramas; changes the appbar to show different possibilities per panorama.
    ''' </summary>
    Private Sub OnPanoramaSelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
        Select Case (CType(sender, Panorama)).SelectedIndex
            ' What's New PanoramaItem
            Case 0
                HubTileService.FreezeGroup("featured")
                ' Featured PanoramaItem
            Case 1
                ApplicationBar.Mode = ApplicationBarMode.Minimized
                ApplicationBar.Buttons.Clear()
                HubTileService.UnfreezeGroup("featured")
                ' All Categories/Feeds PanoramaItem
            Case 2
                ApplicationBar.Mode = ApplicationBarMode.Default
                Dim addButton As ApplicationBarIconButton = New ApplicationBarIconButton With {.Text = AppResources.MainPageAppBarAddText, .IconUri = New Uri("/Icons/appbar.add.rest.png", UriKind.Relative)}
                AddHandler addButton.Click, AddressOf OnAddClick
                ApplicationBar.Buttons.Add(addButton)
                HubTileService.FreezeGroup("featured")
                ' Settings PanoramaItem
            Case 3
                ApplicationBar.Mode = ApplicationBarMode.Minimized
                ApplicationBar.Buttons.Clear()
                HubTileService.FreezeGroup("featured")
        End Select
    End Sub

    ''' <summary>
    ''' Called when the user clicks the refresh appbar menu item; refreshes the application's feeds.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub OnRefreshClick(ByVal sender As Object, ByVal e As EventArgs)
        ' Stop showing progress bar.
        _whatsNewArticles.LoadContent(Sub() LoadingProgressBar.IsIndeterminate = False)
        If Not LoadingProgressBar.IsIndeterminate Then
            LoadingProgressBar.IsIndeterminate = True
        End If
    End Sub

    ''' <summary>
    ''' Called when the user clicks add; Navigates to AddMenu.
    ''' </summary>
    Private Sub OnAddClick(ByVal sender As Object, ByVal e As EventArgs)
        NavigationService.Navigate(New Uri("/Add", UriKind.Relative))
    End Sub

    ''' <summary>
    ''' User has tapped on an article, navigate to article page and show the tapped article.
    ''' </summary>
    Private Sub OnArticleTap(ByVal sender As Object, ByVal e As EventArgs)
        Dim tappedArticle As Article = GetTagAs(Of Article)(sender)
        ' note: We recommend using the sender DataContext instead of GetTagAs. 
        ' See the article for more information: http://go.microsoft.com/fwlink/?LinkId=247592 

        If Nothing IsNot tappedArticle Then
            If Not Convert.ToBoolean(tappedArticle.Read) Then
                tappedArticle.Read = True
                Dim feed As Feed = App.DataBaseUtility.QueryFeed(Convert.ToInt32(tappedArticle.FeedID))
                If Nothing IsNot feed Then
                    feed.UnreadCount -= 1
                    feed.ViewCount += 1
                End If
                App.DataBaseUtility.SaveChangesToDB()
            End If
            NavigationService.Navigate(New Uri("/Article/" & tappedArticle.ArticleID, UriKind.Relative))
        End If
    End Sub

    ''' <summary>
    ''' Called when the user selects remove from the contextmenu.
    ''' </summary>
    Private Sub OnRemoveArticleClick(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Dim selectedArticle As Article = GetTagAs(Of Article)(sender)
        ' note: We recommend using the sender DataContext instead of GetTagAs. 
        ' See the article for more information: http://go.microsoft.com/fwlink/?LinkId=247592

        If Nothing IsNot selectedArticle Then

            If Not Convert.ToBoolean(selectedArticle.Read) Then
                Dim feed As Feed = App.DataBaseUtility.QueryFeed(Convert.ToInt32(selectedArticle.FeedID))
                feed.UnreadCount -= 1
                App.DataBaseUtility.SaveChangesToDB()
            End If

            ' Delete the selected article from the database and remove it from view.
            App.DataBaseUtility.DeleteArticle(selectedArticle)
            _whatsNewArticles.Remove(selectedArticle)
        End If

        ' Reload featured articles in case one of them was deleted.
        _featuredArticles.RedoFeatured()
    End Sub

    ''' <summary>
    ''' Called when the user selects remove on a category context menu.
    ''' </summary>
    Private Sub OnCategoryRemoved(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Dim selectedCategory As Category = GetTagAs(Of Category)(sender)
        ' note: We recommend using the sender DataContext instead of GetTagAs. 
        ' See the article for more information: http://go.microsoft.com/fwlink/?LinkId=247592

        If Nothing IsNot selectedCategory AndAlso 1 <> selectedCategory.CategoryID Then
            ' Delete the selected article from the database and remove it from view.
            App.DataBaseUtility.DeleteCategory(selectedCategory)
            _allCategories.Remove(selectedCategory)
        End If
        ' Reload featured articles in case one of them was deleted.
        _featuredArticles.RedoFeatured()
    End Sub

    ''' <summary>
    ''' Called when the user selects remove on a feed context menu.
    ''' </summary>
    Private Sub OnFeedRemoved(ByVal sender As Object, ByVal e As RoutedEventArgs)
        Dim selectedFeed As Feed = GetTagAs(Of Feed)(sender)
        ' note: We recommend using the sender DataContext instead of GetTagAs. 
        ' See the article for more information: http://go.microsoft.com/fwlink/?LinkId=247592

        If Nothing IsNot selectedFeed Then
            ' Delete the selected article from the database.
            App.DataBaseUtility.DeleteFeed(selectedFeed)
            _allFeeds.RemoveFeed(selectedFeed)
        End If
        ' Reload featured articles in case one of them was deleted.
        _featuredArticles.RedoFeatured()
    End Sub

    ''' <summary>
    ''' User has tapped on a category, navigate to categorypage and show the tapped category with it's corresponding articles.
    ''' </summary>
    Private Sub OnCategoryTap(ByVal sender As Object, ByVal e As EventArgs)
        Dim tappedCategory As Category = GetTagAs(Of Category)(sender)
        ' note: We recommend using the sender DataContext instead of GetTagAs. 
        ' See the article for more information: http://go.microsoft.com/fwlink/?LinkId=247592

        If Nothing IsNot tappedCategory Then
            NavigationService.Navigate(New Uri("/Category/" & tappedCategory.CategoryID, UriKind.Relative))
        End If
    End Sub

    ''' <summary>
    ''' User has tapped on a feed, navigate to feedpage and show the tapped feed with it's corresponding articles.
    ''' </summary>
    Private Sub OnFeedTap(ByVal sender As Object, ByVal e As EventArgs)
        Dim tappedFeed As Feed = GetTagAs(Of Feed)(sender)
        ' note: We recommend using the sender DataContext instead of GetTagAs. 
        ' See the article for more information: http://go.microsoft.com/fwlink/?LinkId=247592

        If Nothing IsNot tappedFeed Then
            NavigationService.Navigate(New Uri("/Feed/" & tappedFeed.FeedID, UriKind.Relative))
        End If
    End Sub

    ''' <summary>
    ''' Called when the user reaches the bottom of the list; load more articles.
    ''' </summary>
    Private Sub OnBottomReached(ByVal sender As Object, ByVal e As EventArgs)
        If Me._whatsNewArticles.Count <> 0 Then
            Dim articles As List(Of Article) = App.DataBaseUtility.NextWhatsNewCollection(_whatsNewArticles.Count)
            If Nothing IsNot articles Then
                For Each a In articles
                    _whatsNewArticles.Add(a)
                Next a
            End If
        End If
    End Sub

    ''' <summary>
    ''' Method called when a HubTile is tapped.
    ''' </summary>
    Private Sub OnHubTileTapped(ByVal sender As Object, ByVal e As System.Windows.Input.GestureEventArgs)
        Dim tappedArticle As Article = GetTagAs(Of Article)(sender)
        ' note: In this case, the sender DataContext is bound to MainPageFeaturedViewModel 
        ' rather than an Article. GetTagAs helps distinguish which Article was tapped.
        ' See the article for more information: http://go.microsoft.com/fwlink/?LinkId=247592

        If Nothing IsNot tappedArticle Then
            NavigationService.Navigate(New Uri("/Article/" & tappedArticle.ArticleID & "/true", UriKind.Relative))
        End If
    End Sub

    ''' <summary>
    ''' Handler called when a user selects the pin to start context menu option on a category.
    ''' </summary>
    ''' <param name="sender">The sender of this event.</param>
    ''' <param name="e">The event arguments for this event.</param>
    Private Sub OnCategoryPinned(ByVal sender As Object, ByVal e As EventArgs)
        Dim tappedCategory As Category = GetTagAs(Of Category)(sender)
        ' note: We recommend using the sender DataContext instead of GetTagAs. 
        ' See the article for more information: http://go.microsoft.com/fwlink/?LinkId=247592

        If Nothing IsNot tappedCategory Then
            AddTile.AddLiveTile(tappedCategory)
        End If

    End Sub

    ''' <summary>
    ''' Handler called when a user selects the pin to start context menu option on a feed.
    ''' </summary>
    ''' <param name="sender">The sender of this event.</param>
    ''' <param name="e">The event arguments for this event.</param>
    Private Sub OnFeedPinned(ByVal sender As Object, ByVal e As EventArgs)
        Dim tappedFeed As Feed = GetTagAs(Of Feed)(sender)
        ' note: We recommend using the sender DataContext instead of GetTagAs. 
        ' See the article for more information: http://go.microsoft.com/fwlink/?LinkId=247592

        If Nothing IsNot tappedFeed Then
            AddTile.AddLiveTile(tappedFeed)
        End If
    End Sub

    ''' <summary>
    ''' Gets the tag property of the given container, given that the container is a FrameWorkElement.
    ''' </summary>
    ''' <typeparam name="T">The type to return the tag property as.</typeparam>
    ''' <param name="container">The FramworkElement (as an object) to get the tag from.</param>
    ''' <returns></returns>
    Public Function GetTagAs(Of T As Class)(ByVal container As Object) As T
        If Nothing IsNot container Then
            Dim element As FrameworkElement = TryCast(container, FrameworkElement)
            If Nothing IsNot element Then
                Dim tag As T = TryCast(element.Tag, T)
                If Nothing IsNot tag Then
                    Return tag
                End If
            End If
        End If
        Return Nothing
    End Function
End Class
