﻿' 
'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  

Imports System.Windows.Navigation
Imports FeedCastLibrary
Imports FeedCast.Resources

Namespace Views
    Partial Public Class NewCategoryPage
        Inherits PhoneApplicationPage

        ''' <summary>
        ''' Application bar save button
        ''' </summary>
        Private _saveButton As ApplicationBarIconButton

        Public Sub New()
            InitializeComponent()
        End Sub

        ''' <summary>
        ''' Called when the application is navigated to.
        ''' </summary>
        Protected Overrides Sub OnNavigatedTo(ByVal e As NavigationEventArgs)
            MyBase.OnNavigatedTo(e)

            ' Setting up applicationbar for proper loading and localization.
            Me.ApplicationBar = New ApplicationBar
            _saveButton = New ApplicationBarIconButton With {.IconUri = New Uri("/Icons/appbar.save.rest.png", UriKind.Relative), .Text = AppResources.NewCategoryAppBarSaveText, .IsEnabled = False}
            AddHandler _saveButton.Click, AddressOf OnSaveClick
            Me.ApplicationBar.Buttons.Add(_saveButton)

            ' Setting DataContext.
            FeedPicker.DataContext = App.DataBaseUtility.GetAllFeeds
        End Sub

        ''' <summary>
        ''' User has clicked save. Category is added to database along with it's associated feeds.
        ''' </summary>
        Private Sub OnSaveClick(ByVal sender As Object, ByVal e As EventArgs)
            Dim text As String = CategoryNameTextBox.Text
            If Not String.IsNullOrWhiteSpace(text) Then
                Dim newCat As New Category With {.CategoryTitle = text}
                App.DataBaseUtility.AddCategory(newCat)

                App.DataBaseUtility.SaveChangesToDB()

                For Each f In FeedPicker.SelectedItems
                    App.DataBaseUtility.AddFeed(f, newCat)
                Next f

                App.DataBaseUtility.SaveChangesToDB()

                If NavigationService.CanGoBack Then
                    NavigationService.GoBack()
                End If
            End If
        End Sub

        ''' <summary>
        ''' Check if the user presses enter to save the category.
        ''' </summary>
        Private Sub OnTextBoxKeyUp(ByVal sender As Object, ByVal e As KeyEventArgs)
            If Not String.IsNullOrWhiteSpace(CategoryNameTextBox.Text) Then
                _saveButton.IsEnabled = True
                If Nothing IsNot e AndAlso e.Key = Key.Enter Then
                    OnSaveClick(sender, e)
                End If
            Else
                _saveButton.IsEnabled = False
            End If
        End Sub
    End Class
End Namespace