﻿' 
'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  
'

Imports System.Collections
Imports System.Collections.ObjectModel
Imports FeedCast.Resources
Imports FeedCast.ViewModels
Imports FeedCastLibrary

Namespace Views
    ''' <summary>
    ''' Page a user navigates to when they would like to add a new feed to their reader.
    ''' </summary>
    Partial Public Class NewFeedPage
        Inherits PhoneApplicationPage

        ''' <summary>
        ''' Collection of search results that gets populated once the user performs a search.
        ''' </summary>
        Private _search As NewFeedPageViewModel

        Public Sub New()
            InitializeComponent()

            _search = New NewFeedPageViewModel
        End Sub

        ''' <summary>
        ''' User has navigated to this page from the AddMenu; they have decided to add a new feed.
        ''' </summary>
        Protected Overrides Sub OnNavigatedTo(ByVal e As NavigationEventArgs)
            MyBase.OnNavigatedTo(e)

            ' Loading application bar form code behind for propert timing and localization.
            Me.ApplicationBar = New ApplicationBar
            Dim saveButton As ApplicationBarIconButton = New ApplicationBarIconButton With {.IconUri = New Uri("/Icons/appbar.save.rest.png", UriKind.Relative), .Text = AppResources.NewCategoryAppBarSaveText, .IsEnabled = False}
            AddHandler saveButton.Click, AddressOf OnSaveClick
            Me.ApplicationBar.Buttons.Add(saveButton)

            Dim searchButton As ApplicationBarIconButton = New ApplicationBarIconButton With {.IconUri = New Uri("/Icons/appbar.feature.search.rest.png", UriKind.Relative), .Text = AppResources.NewFeedAppBarSearchText}
            AddHandler searchButton.Click, AddressOf OnSearchClick
            Me.ApplicationBar.Buttons.Add(searchButton)

            ' Setting page datacontext.
            SearchResults.DataContext = _search
            Dim categories As List(Of Category) = App.DataBaseUtility.GetAllCategories
            categories.Remove(App.Favorites)
            CategoryPicker.DataContext = categories

            Dim idStr As String = Nothing
            If NavigationContext.QueryString.TryGetValue("selectedCatID", idStr) Then
                Dim catID As Integer = Convert.ToInt32(idStr)
                Dim selectedCat As Category = App.DataBaseUtility.QueryCategory(catID)
                If Nothing IsNot selectedCat AndAlso CategoryPicker.Items.Contains(selectedCat) Then
                    CategoryPicker.SelectedItem = selectedCat
                End If
            End If
        End Sub

        ''' <summary>
        ''' User clicks the search button. Search results are populated and the user
        ''' is provided with a checkbox appbar button to finalize their selection.
        ''' </summary>
        Private Sub OnSearchClick(ByVal sender As Object, ByVal e As EventArgs)
            ' Check to make sure the user had not initiated a download.
            If Not _search.IsDownloading Then
                ' Dismiss focus on the keyboard (hide keyboard).
                Me.Focus()

                ' Search for the user's query.
                Dim query As String = SearchBox.Text
                If Not String.IsNullOrWhiteSpace(query) Then
                    ' Remove the no results text if present.
                    NoResultsText.Visibility = Visibility.Collapsed

                    ' Show progress bar.
                    LoadingProgressBar.IsIndeterminate = True

                    ' Search for results.
                    ' Stop showing progress bar.
                    ' If there are no search results show NoResultsText, else hide it.
                    _search.GetResults(query, Sub(count)
                                                  LoadingProgressBar.IsIndeterminate = False
                                                  NoResultsText.Visibility = If(count <= 0, Visibility.Visible, Visibility.Collapsed)
                                              End Sub)
                Else
                    ' User has entered an invalid query.
                    NoResultsText.Visibility = Visibility.Visible
                End If
            End If
        End Sub

        ''' <summary>
        ''' Check if the user intended to begin the search by hitting the enter key.
        ''' </summary>
        Private Sub OnTextBoxKeyUp(ByVal sender As Object, ByVal e As KeyEventArgs)
            ' Check if the user pressed the enter key to begin the search.
            If Nothing IsNot e AndAlso e.Key = Key.Enter Then
                OnSearchClick(sender, e)
            End If
        End Sub

        ''' <summary>
        ''' User has selected a category.
        ''' </summary>
        Private Sub OnSaveClick(ByVal sender As Object, ByVal e As EventArgs)
            Dim selectedFeeds As IList = SearchResults.SelectedItems
            If Nothing IsNot selectedFeeds AndAlso selectedFeeds.Count > 0 Then
                Dim selectedCategory As Category = TryCast(CategoryPicker.SelectedItem, Category)
                If Nothing IsNot selectedCategory Then
                    Dim selectedItems As New Collection(Of Article)

                    For Each a In SearchResults.SelectedItems
                        selectedItems.Add(a)
                    Next a

                    If Nothing IsNot selectedItems Then
                        _search.ResultToFeed(selectedCategory, New Collection(Of Article)(selectedItems))
                    End If

                    If NavigationService.CanGoBack Then
                        NavigationService.GoBack()
                    End If
                End If
            End If
        End Sub

        ''' <summary>
        ''' 
        ''' </summary>
        Private Sub OnSearchResultsSelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
            Dim saveButton As ApplicationBarIconButton = TryCast(ApplicationBar.Buttons(0), ApplicationBarIconButton)

            If e.AddedItems.Count > 0 Then
                If Nothing IsNot saveButton Then
                    saveButton.IsEnabled = True
                End If
            Else
                saveButton.IsEnabled = False
            End If
        End Sub
    End Class
End Namespace