﻿' 
'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  
'

'#define DEBUG_AGENT

Imports FeedCastLibrary
Imports Microsoft.Phone.Scheduler

Public Class ScheduledAgent
    Inherits ScheduledTaskAgent

    ''' <summary>
    ''' Tracks whether the agent is initialized or not
    ''' </summary>
    Private Shared _classInitialized As Boolean

    ''' <summary>
    ''' Stores the location of the database
    ''' </summary>
    Private Shared ReadOnly DBLocation As String = "Data Source=isostore:/LocalDatabase.sdf"

    ''' <summary>
    ''' DataBaseUtility allowing the agent use with the database
    ''' </summary>
    Private Shared _dataBaseTools As DataUtils
    Protected Shared Property DataBaseTools As DataUtils
        Get
            Return _dataBaseTools
        End Get
        Private Set(ByVal value As DataUtils)
            _dataBaseTools = value
        End Set
    End Property

    ''' <summary>
    ''' Allows the background agent access into isolated storage
    ''' </summary>
    Private AgentSettings As Settings

    ''' <summary>
    ''' Holds the number of remaining downloads
    ''' </summary>
    Private _remainingDownloads As Integer

    ''' <remarks>
    ''' ScheduledAgent constructor, initializes the UnhandledException handler
    ''' </remarks>
    Public Sub New()
        If Not _classInitialized Then
            _classInitialized = True
            ' Subscribe to the managed exception handler
            Deployment.Current.Dispatcher.BeginInvoke(Sub() AddHandler Application.Current.UnhandledException, AddressOf ScheduledAgent_UnhandledException)
        End If

        ' Initialize the database utilities and storage settings.
        Dim db As New LocalDatabaseDataContext(DBLocation)
        DataBaseTools = New DataUtils(DBLocation)
        AgentSettings = New Settings
    End Sub

    ''' Code to execute on Unhandled Exceptions
    Private Sub ScheduledAgent_UnhandledException(ByVal sender As Object, ByVal e As ApplicationUnhandledExceptionEventArgs)
        If System.Diagnostics.Debugger.IsAttached Then
            ' An unhandled exception has occurred; break into the debugger
            System.Diagnostics.Debugger.Break()
        End If
    End Sub

    ''' <summary>
    ''' Agent that runs a scheduled task
    ''' </summary>
    ''' <param name="task">
    ''' The invoked task
    ''' </param>
    ''' <remarks>
    ''' This method is called when a periodic or resource intensive task is invoked
    ''' </remarks>
    Protected Overrides Sub OnInvoke(ByVal task As ScheduledTask)
        ' Run the periodic task.
        Dim allFeeds As List(Of Feed) = DataBaseTools.GetAllFeeds
        _remainingDownloads = allFeeds.Count
        If _remainingDownloads > 0 Then
            'TODO handle errors.
            Deployment.Current.Dispatcher.BeginInvoke(Sub()
                                                          Dim downloader As New WebTools(New SynFeedParser)
                                                          AddHandler downloader.SingleDownloadFinished, AddressOf SendToDatabase
                                                          Try
                                                              downloader.Download(allFeeds)
                                                          Catch
                                                          End Try
                                                      End Sub)
        End If

        ' Used to quickly invoke task when debugging.
        '#If DEBUG_AGENT Then
        'ScheduledActionService.LaunchForTest(task.Name, System.TimeSpan.FromSeconds(600))
        '#End If
    End Sub

    ''' <summary>
    ''' Send the articles that the background agent has downloaded to the database.
    ''' </summary>
    ''' <param name="sender">The object that calls the event</param>
    ''' <param name="e">The value returned from the event raiser</param>
    Private Sub SendToDatabase(ByVal sender As Object, ByVal e As SingleDownloadFinishedEventArgs)
        ' Make sure download is not null!
        If e.DownloadedArticles IsNot Nothing Then
            DataBaseTools.AddArticles(e.DownloadedArticles, e.ParentFeed)
            _remainingDownloads -= 1
            'System.Diagnostics.Debug.WriteLine("BGAgent downloaded " & e.ParentFeed.FeedTitle)
        End If

        ' If there are no remaining downloads, 
        ' tell the scheduler that the background agent is done.
        If _remainingDownloads <= 0 Then
            NotifyComplete()
        End If
    End Sub
End Class