﻿'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  


Imports System.ComponentModel
Imports System.Threading
Imports System.Data.Linq
Imports System.Data.Linq.Mapping

Public Class DataUtils
    Implements INotifyPropertyChanged

    'Database!
    Private db As LocalDatabaseDataContext

    Private dbMutex As New Mutex(False, "DBControl")
    ' note: could also have used a lock rather than a mutex.
    ' A lock protects against cross-thread access to the data context.
    ' The mutex additionally protects against cross-process access.

    'Contructor to initialize the database
    Public Sub New(ByVal dbConnectionString As String)
        db = New LocalDatabaseDataContext(dbConnectionString)
    End Sub

    'Used when the database is modified.
    Public Sub SaveChangesToDB()
        dbMutex.WaitOne()

        Try
            ' Attempt all updates.
            db.SubmitChanges(ConflictMode.ContinueOnConflict)
        Catch e1 As ChangeConflictException
            '  For debugging.
            'System.Diagnostics.Debug.WriteLine("Optimistic concurrency error.")
            'System.Diagnostics.Debug.WriteLine(e.Message)
            'For Each occ As ObjectChangeConflict In db.ChangeConflicts
            '    Dim metatable As MetaTable = db.Mapping.GetTable(occ.Object.GetType)
            '    System.Diagnostics.Debug.WriteLine("Table name: {0}", metatable.TableName)
            '    For Each mcc As MemberChangeConflict In occ.MemberConflicts
            '        Dim currVal As Object = mcc.CurrentValue
            '        Dim origVal As Object = mcc.OriginalValue
            '        Dim databaseVal As Object = mcc.DatabaseValue

            '        System.Diagnostics.Debug.WriteLine("current value: {0}", currVal)
            '        System.Diagnostics.Debug.WriteLine("original value: {0}", origVal)
            '        System.Diagnostics.Debug.WriteLine("database value: {0}", databaseVal)
            '    Next mcc
            'Next occ

            For Each occ In db.ChangeConflicts
                ' Keep the value that has changed, update the other values with database values.
                occ.Resolve(RefreshMode.KeepChanges)
            Next occ
        Catch e As Exception
            System.Diagnostics.Debug.WriteLine("ERROR!!!: " & e.ToString)
            Exit Sub
            ' Minimize wait for other threads.
        Finally
            dbMutex.ReleaseMutex()
        End Try
    End Sub

    'Notify Silverlight to update the UI 
    ' if the database changes
#Region "INotifyProperyChangedMembers"
    Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

    Private Sub NotifyPropertyChanged(ByVal propertyName As String)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub
#End Region

#Region "Featured"
    ''' <summary>
    ''' Queries especially for Featured. 
    ''' </summary>
    Public Function QueryForUnreadFeeds() As List(Of Feed)
        dbMutex.WaitOne()

        ' Query the database for feeds with unread articles.
        Dim c = From q In db.Feed
                Where q.UnreadCount <> 0
                Order By q.ViewCount Descending
                Select q
        dbMutex.ReleaseMutex()
        Return c.ToList
    End Function

    Public Function QueryForSortedFeed(ByVal c As List(Of Feed), ByVal maxThresIndexes() As Integer, ByVal index As Integer) As List(Of Article)
        dbMutex.WaitOne()

        ' Query for the newest article of the chosen feed.
        Dim a = From o In db.Article
                Where o.FeedID = c(maxThresIndexes(index)).FeedID
                Where o.Read = False
                Order By o.PublishDate Descending
                Select o
        dbMutex.ReleaseMutex()
        Return a.ToList
    End Function
#End Region

    'Check if the feed is there before adding
    Public Function checkFeed(ByVal feedURI As String, ByVal catID As Integer) As Boolean
        Dim result As Boolean
        Dim cat_feed As New List(Of Category_Feed)
        Dim c = From q In db.Feed
                Where q.FeedBaseURI = feedURI
                Select q

        For Each f As Feed In c
            Dim ff As Feed = f
            Dim d = From q In db.Category_Feed
                    Where q.CategoryID = catID
                    Where q.FeedID = ff.FeedID
                    Select q
            cat_feed = d.ToList
        Next f
        result = (cat_feed.Count = 0)
        Return result
    End Function

    'Check if the feed is there before adding
    Public Function checkIfFeedExists(ByVal feedURI As String) As Boolean
        Dim feed As New List(Of Feed)
        Dim c = From q In db.Feed
                Where q.FeedBaseURI = feedURI
                Select q
        feed = c.ToList
        Return (feed.Count = 0)
    End Function

    'Intial List of What's New articles, without a limiter
    Public Function InitialWhatsNewCollection() As List(Of Article)
        dbMutex.WaitOne()

        Dim w As New List(Of Article)
        Dim whatsnew = From q As Article In db.Article
                       Where (q.PublishDate).Value.CompareTo(Date.Now) < 0
                       Where q.PublishDate.Value.AddDays(3).CompareTo(Date.Now) > 0
                       Order By q.PublishDate Descending
                       Select q
        w = whatsnew.ToList
        If w.Count <> 0 Then
            SynFeedParser.latestDate = w(0).PublishDate.Value
        End If
        dbMutex.ReleaseMutex()
        Return w
    End Function

    'Intial List of What's New articles
    Public Function WhatsNewCollection(ByVal initialCount As Integer) As List(Of Article)
        dbMutex.WaitOne()
        Dim w As New List(Of Article)
        Dim whatsnew = From q As Article In db.Article
                       Where (q.PublishDate).Value.CompareTo(Date.Now) < 0
                       Where q.PublishDate.Value.AddDays(3).CompareTo(Date.Now) > 0
                       Order By q.PublishDate Descending
                       Select q
        w = whatsnew.ToList
        If w.Count <> 0 Then
            If w.Count >= initialCount Then
                w.RemoveRange(initialCount, w.Count - initialCount)
            End If
            SynFeedParser.latestDate = w(0).PublishDate.Value
        End If
        dbMutex.ReleaseMutex()
        Return w
    End Function

    'Articles that have been downloaded since the last refresh
    Public Function UpdateWhatsNewCollection() As List(Of Article)
        dbMutex.WaitOne()
        Dim w As New List(Of Article)
        Dim whatsnew = From q As Article In db.Article
                       Where q.PublishDate.Value.ToLocalTime.CompareTo(SynFeedParser.latestDate) > 0
                       Order By q.PublishDate Ascending
                       Select q
        Try
            w = whatsnew.ToList
            If w.Count <> 0 AndAlso whatsnew IsNot Nothing Then
                SynFeedParser.latestDate = w(w.Count - 1).PublishDate.Value
            End If
        Catch
            Return w

        Finally
            dbMutex.ReleaseMutex()
        End Try
        Return w
    End Function

    'When the user hits the bottom, the application is supposed to load more articles
    Public Function NextWhatsNewCollection(ByVal count As Integer) As List(Of Article)
        dbMutex.WaitOne()

        Dim w As New List(Of Article)
        Dim whatsNew As New List(Of Article)
        Dim whatsnew_Renamed = From q As Article In db.Article
                               Where q.PublishDate.Value.AddDays(3).CompareTo(Date.Now) > 0
                               Order By q.PublishDate Descending
                               Select q

        w = whatsnew_Renamed.ToList
        If w.Count > 0 Then
            If w.Count > count Then
                If w.Count - count > 10 Then
                    whatsNew = w.ToList.GetRange(count, 10).ToList
                Else
                    whatsNew = w.ToList.GetRange(count, w.Count - count).ToList
                End If

            Else
                whatsNew = Nothing
            End If
        End If
        dbMutex.ReleaseMutex()
        Return whatsNew
    End Function

    'Add a category to the category table
    Public Function AddCategory(ByVal newCategory As Category) As Boolean
        dbMutex.WaitOne()

        Dim c = From q In db.Category
                Where q.CategoryTitle = newCategory.CategoryTitle
                Select q
        Dim checkCategory As Boolean = (c.Count() = 0)
        If checkCategory Then
            newCategory.IsPinned = False
            If newCategory.CategoryID <> 1 Then
                newCategory.IsRemovable = True
            Else
                newCategory.IsRemovable = False
            End If

            db.Category.InsertOnSubmit(newCategory)
            SaveChangesToDB()
        End If

        dbMutex.ReleaseMutex()
        Return checkCategory
    End Function

    'Add a feed to the feed table, and a corresponding entry in Category_Feed Table
    Public Function AddFeed(ByVal newFeed As Feed, ByVal cat As Category) As Boolean
        dbMutex.WaitOne()

        Dim checkfeed As Boolean = Me.checkFeed(newFeed.FeedBaseURI, cat.CategoryID)
        Dim feedexists As Boolean = checkIfFeedExists(newFeed.FeedBaseURI)

        'Checks if the feed is in the database
        If feedexists Then
            newFeed.SharedCount = 0
            newFeed.UnreadCount = 0
            newFeed.ViewCount = 0
            newFeed.FavoritedCount = 0
            newFeed.IsPinned = False
            db.Feed.InsertOnSubmit(newFeed)
            SaveChangesToDB()
            Dim newCat_Feed As Category_Feed = New Category_Feed With {.CategoryID = cat.CategoryID, .FeedID = newFeed.FeedID}
            db.Category_Feed.InsertOnSubmit(newCat_Feed)
            SaveChangesToDB()

            'If it does, but not to the category, add it to the category
        ElseIf checkfeed Then
            Dim newCat_Feed As Category_Feed = New Category_Feed With {.CategoryID = cat.CategoryID, .FeedID = newFeed.FeedID}
            db.Category_Feed.InsertOnSubmit(newCat_Feed)
            SaveChangesToDB()
        End If
        'Else do nothing.
        dbMutex.ReleaseMutex()
        Return (feedexists)
    End Function

    Public Sub AddCat_Feed(ByVal newFeed As Integer, ByVal cat As Integer)
        Dim newCat_Feed As Category_Feed = New Category_Feed With {.CategoryID = cat, .FeedID = newFeed}
        db.Category_Feed.InsertOnSubmit(newCat_Feed)
        SaveChangesToDB()
    End Sub

    'Add an article to the article table!
    Public Sub AddArticles(ByVal newArticles As ICollection(Of Article), ByVal feed As Feed)
        dbMutex.WaitOne()

        Dim downloadedArticleCount As Integer = newArticles.Count
        Dim numOfNew As Integer = 0

        ' Query local database for existing articles.
        Dim i As Integer = 0
        Do While i < downloadedArticleCount
            Dim newArticle As Article = newArticles.ElementAt(i)
            Dim d = From q In db.Article
                    Where q.ArticleBaseURI = newArticle.ArticleBaseURI
                    Select q

            Dim a As List(Of Article) = d.ToList

            'Determine if any articles are already in the database 
            Dim alreadyInDB As Boolean = (d.ToList.Count = 0)
            If alreadyInDB Then
                newArticle.Read = False
                newArticle.Favorite = False
                numOfNew += 1
            Else
                'If so, remove them from the list
                newArticles.Remove(newArticle)
                downloadedArticleCount -= 1
                i -= 1
            End If

            i += 1
        Loop
        'Try to submit and update counts. 
        Try
            db.Article.InsertAllOnSubmit(newArticles)
            Deployment.Current.Dispatcher.BeginInvoke(Sub()
                                                          feed.UnreadCount += numOfNew
                                                          SaveChangesToDB()
                                                      End Sub)
            SaveChangesToDB()
            'TODO handle errors.
        Catch
        Finally
            dbMutex.ReleaseMutex()
        End Try
    End Sub

    'Corner Case (if the feed doesnt belong to any other category)
    Public Function checkDelete(ByVal feed As Integer) As Boolean
        Dim d = From o In db.Category_Feed
                Where o.FeedID = feed
                Select o
        Dim cat_feed As List(Of Category_Feed) = d.ToList
        Dim i As Integer = 0
        Do While i < cat_feed.Count
            If cat_feed(i).CategoryID = 1 Then
                cat_feed.RemoveAt(i)
            End If
            i += 1
        Loop
        Return (cat_feed.Count = 1)
    End Function

    'Delete a category.
    Public Sub DeleteCategory(ByVal deleteCategory As Category)
        'If the category is not favorites
        If deleteCategory.CategoryID <> 1 Then
            'Get all the pairings of the category
            Dim q = From o In db.Category_Feed
                    Where o.CategoryID = deleteCategory.CategoryID
                    Select o
            'Foreach of the pairings
            For Each deleteCat_Feed In q
                'Check if the feed has any other association 
                If checkDelete(Convert.ToInt32(deleteCat_Feed.FeedID)) Then
                    'If not, delete it.
                    Dim feed As Feed = QueryFeed(Convert.ToInt32(deleteCat_Feed.FeedID))
                    DeleteFeed(feed)
                End If
            Next deleteCat_Feed

            'Remove the cat_feeds even if they are associated with Favorites Category. 
            Dim catfeed As List(Of Category_Feed) = q.ToList
            Dim max As Integer = catfeed.Count
            Dim i As Integer = 0
            Do While i < max
                If catfeed(i).CategoryID = 1 Then
                    catfeed.RemoveAt(i)
                    max -= 1
                    i -= 1
                End If
                i += 1
            Loop
            db.Category_Feed.DeleteAllOnSubmit(catfeed)
            db.Category.DeleteOnSubmit(deleteCategory)
            SaveChangesToDB()
        End If
    End Sub

    'Delete a feed.
    Public Sub DeleteFeed(ByVal deleteFeed As Feed)
        Dim q = From o In db.Category_Feed
                Where o.FeedID = deleteFeed.FeedID
                Select o
        'Delete all the cat_feed pairings 
        db.Category_Feed.DeleteAllOnSubmit(q.ToList)
        Dim a = From o In db.Article
                Where o.FeedID = deleteFeed.FeedID
                Select o
        'Delete all the articles in that feed
        DeleteArticles(a.ToList)
        db.Feed.DeleteOnSubmit(deleteFeed)
        SaveChangesToDB()
    End Sub

    'Delete an article.
    Public Sub DeleteArticle(ByVal deleteArticle As Article)
        db.Article.DeleteOnSubmit(deleteArticle)
        SaveChangesToDB()
    End Sub

    'Delete articles
    Public Sub DeleteArticles(ByVal deleteArticles As List(Of Article))
        Dim max As Integer = deleteArticles.Count
        Dim i As Integer = 0
        Do While i < max
            'If favorited remove it from the collection to be deleted
            If Convert.ToBoolean(deleteArticles(i).Favorite) Then
                deleteArticles.RemoveAt(i)
                max -= 1
                i -= 1
            End If
            i += 1
        Loop
        'Delete all the one not favorited. 
        db.Article.DeleteAllOnSubmit(deleteArticles)
        SaveChangesToDB()
    End Sub

    'Query for a particular category
    Public Function QueryCategory(ByVal category As Integer) As Category
        Dim result As Category = Nothing
        Dim c = From q As Category In db.Category
                Where q.CategoryID = category
                Select q
        If c.Count() > 0 Then
            result = c.Take(1).Single
        End If
        Return result
    End Function

    'Query for a particular feed
    Public Function QueryFeed(ByVal feed As Integer) As Feed
        Dim result As Feed = Nothing
        Dim c = From q As Feed In db.Feed
                Where q.FeedID = feed
                Select q
        If c.Count() > 0 Then
            result = c.Take(1).Single
        End If
        Return result
    End Function

    'Query for a particular article
    Public Function QueryArticle(ByVal articleID As Integer) As Article
        Dim result As Article = Nothing
        Dim c = From q As Article In db.Article
                Where q.ArticleID = articleID
                Select q
        If c.Count() > 0 Then
            result = c.Take(1).Single
        End If
        Return result
    End Function

    'Get all the Categories
    Public Function GetAllCategories() As List(Of Category)
        dbMutex.WaitOne()
        Dim cat As List(Of Category)
        Dim c = From q In db.Category
                Select q
        cat = c.ToList
        dbMutex.ReleaseMutex()
        Return cat
    End Function

    'Get all the Feeds
    Public Function GetAllFeeds() As List(Of Feed)
        dbMutex.WaitOne()
        Dim f As List(Of Feed)
        Dim c = From q In db.Feed
                Select q
        f = c.ToList
        dbMutex.ReleaseMutex()
        Return f
    End Function

    'Get all the Articles no filter
    Public Function GetAllArticles() As List(Of Article)
        dbMutex.WaitOne()
        Dim a As List(Of Article)
        Dim c = From q In db.Article
                Order By q.PublishDate Descending
                Select q
        a = c.ToList
        dbMutex.ReleaseMutex()
        Return a
    End Function

    'Get all the Articles filtered by feed
    Public Function GetFeedArticles(ByVal feedID As Integer) As List(Of Article)
        Dim a As List(Of Article)
        Dim q = From o As Article In db.Article
                Where o.FeedID = feedID
                Order By o.PublishDate Descending
                Select o

        a = q.ToList

        Return a
    End Function

    'Get all the feeds in a category (Used to remove off the UI)
    Public Function GetFeeds(ByVal catID As Integer) As List(Of Feed)
        Dim feeds As New List(Of Feed)
        Dim c = From o In db.Category_Feed
                Where o.CategoryID = catID
                Select o
        For Each cat In c.ToList
            If checkDelete(Convert.ToInt32(cat.FeedID)) Then
                feeds.Add(QueryFeed(Convert.ToInt32(cat.FeedID)))
            End If
        Next cat
        Return feeds
    End Function

    'Get all the Articles filtered by Category
    Public Function GetCategoryArticles(ByVal categoryID As Integer) As List(Of Article)
        Dim cat As New List(Of Article)
        'Favorites?
        If categoryID = 1 Then
            Dim o = From a In db.Article
                    Where a.Favorite = True
                    Select a
            cat.AddRange(o.ToList)
        Else
            'Get the corresponding Cat_Feeds with the category ID
            Dim q = From o In db.Category_Feed
                    Where o.CategoryID = categoryID
                    Select o
            ' Not Favorites!
            Dim feedArticles As List(Of Article)
            For Each f In q
                ' For every Cat_Feed get articles
                feedArticles = GetFeedArticles(Convert.ToInt32(f.FeedID))
                cat.AddRange(feedArticles)
            Next f
        End If

        cat.Sort()
        cat.Reverse()
        'Return list.
        Return cat
    End Function

    Public Sub clearOldArticles()
        dbMutex.WaitOne()
        Dim o = From a In db.Article
                Where a.PublishDate.Value.AddDays(7).CompareTo(Date.Now) < 0
                Where a.Favorite = False
                Where a.Read = True
                Select a

        Dim j = From a In db.Article
                Where a.PublishDate.Value.AddDays(14).CompareTo(Date.Now) < 0
                Where a.Favorite = False
                Select a

        If Nothing IsNot o AndAlso o.ToList.Count <> 0 Then
            Dim allArticles As List(Of Article) = o.ToList
            allArticles.AddRange(j.ToList)

            For Each a In allArticles
                Dim feed As Feed = QueryFeed(Convert.ToInt32(a.FeedID))
                Deployment.Current.Dispatcher.BeginInvoke(Sub()
                                                              feed.UnreadCount -= 1
                                                              SaveChangesToDB()
                                                          End Sub)
                Dim unread As Integer = Convert.ToInt32(feed.UnreadCount)
            Next a
            DeleteArticles(o.ToList)
        End If
        dbMutex.ReleaseMutex()
    End Sub

    Public Sub PrintOutput()
        dbMutex.WaitOne()
        Dim a As List(Of Article)
        Dim f As List(Of Feed)
        Dim c As List(Of Category)
        Dim o = From q In db.Article
                Select q
        a = o.ToList
        Dim l = From q In db.Feed
                Select q
        f = l.ToList
        Dim m = From q In db.Category
                Select q
        c = m.ToList

        For Each article In a
            System.Diagnostics.Debug.WriteLine(article.ArticleTitle)
            System.Diagnostics.Debug.WriteLine(article.ArticleID)
        Next article

        For Each feed In f
            System.Diagnostics.Debug.WriteLine(feed.FeedTitle)
            System.Diagnostics.Debug.WriteLine(feed.FeedID)
        Next feed

        For Each category In c
            System.Diagnostics.Debug.WriteLine(category.CategoryTitle)
            System.Diagnostics.Debug.WriteLine(category.CategoryID)
        Next category
        dbMutex.ReleaseMutex()
    End Sub
End Class
