﻿' 
'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  
'

Partial Public Class Feed
    Implements IComparable(Of Feed), IEquatable(Of Feed)

    ''' <summary>
    ''' Get the corresponding name group key for this feed.
    '''  <param name="feed">The feed whose group key to retrieve.</param>
    ''' <returns>The group key associated with this feed name.
    ''' Ex: a feed called "Microsoft" returns "m".</returns>
    ''' </summary>
    Public Function GetNameKey() As String
        Return Me.FeedTitle(0).ToString.ToLower
    End Function

    ''' <summary>
    ''' Compares this instance with the specified feed and indicates
    ''' whether this instance precedes, follows, or appears in the 
    ''' same position in the sort order as the specified feed.
    ''' </summary>
    ''' <param name="other"></param>
    ''' <returns>Less than zero if this instance precedes "other".
    ''' Zero if this instance has the same position in the sort order as "other".
    ''' Greater than 0 if This instance follows value or value is null.</returns>
    Public Function CompareTo(ByVal other As Feed) As Integer Implements IComparable(Of Feed).CompareTo
        Return Me.FeedTitle.CompareTo(other.FeedTitle)
    End Function

    ''' <summary>
    ''' Determines whether two Feed objects have the same value.
    ''' </summary>
    ''' <param name="other"> The feed to compare this feed to.</param>
    ''' <returns>True if this feed has the same value as "other" feed,
    ''' otherwise false.</returns>
    Public Overloads Function Equals(ByVal other As Feed) As Boolean Implements IEquatable(Of Feed).Equals
        Return Me.FeedBaseURI.Equals(other.FeedBaseURI)
    End Function

    ''' <summary>
    ''' Returns a string that represents this feed.
    ''' </summary>
    ''' <returns>A string representation of this feed.</returns>
    Public Overrides Function ToString() As String
        Return Me.FeedTitle
    End Function
End Class
