﻿' 
'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  
'

Imports System.Collections.ObjectModel
Imports System.Diagnostics
Imports System.IO.IsolatedStorage

' TODO move from ApplicationSettings to Isolated storage to improve performance
Public Class Settings
    ' Isolated storage settings
    Private Shared _isolatedStore As IsolatedStorageSettings

    ' Isolated storage key names
    Private Const WifiOnlySettingKeyName As String = "WifiOnlySetting"
    Private Const InitialLaunchSettingKeyName As String = "InitialLaunchSetting"
    Private Const FeaturedArticlesKeyName As String = "FeaturedArticles"
    Private Const LastUpdatedTimeKeyName As String = "LastUpdatedTime"

    ' Default values of our settings
    Private Const WifiOnlySettingDefault As Boolean = False
    Private Const InitialLaunchSettingDefault As Boolean = True

    Private Shared ReadOnly FeaturedArticlesDefault As Collection(Of FeedCastLibrary.Article) = Nothing

    Private Const LastUpdatedDefault As String = Nothing

    ''' <summary>
    ''' Constructor that gets the application settings.
    ''' </summary>
    Public Sub New()
        Try
            ' Get previous application settings.
            _isolatedStore = IsolatedStorageSettings.ApplicationSettings
        Catch e As Exception
            Debug.WriteLine("Exception while using IsolatedStorageSettings: " & e.ToString)
        End Try
    End Sub

    ''' <summary>
    ''' Update a setting value for our application. If the setting does not
    ''' exist, then add the setting.
    ''' </summary>
    ''' <param name="Key"></param>
    ''' <param name="value"></param>
    ''' <returns></returns>
    Private Shared Function AddOrUpdateValue(ByVal Key As String, ByVal value As Object) As Boolean
        Dim valueChanged As Boolean = False

        ' If the key exists
        If _isolatedStore.Contains(Key) Then
            ' If the value has changed
            If _isolatedStore(Key) IsNot value Then
                ' Store the new value
                _isolatedStore(Key) = value
                valueChanged = True
            End If
            ' Otherwise create the key.
        Else
            _isolatedStore.Add(Key, value)
            valueChanged = True
        End If

        Return valueChanged
    End Function

    ''' <summary>
    ''' Get the current value of the setting, or if it is not found, set the 
    ''' setting to the default setting.
    ''' </summary>
    ''' <typeparam name="valueType"></typeparam>
    ''' <param name="Key"></param>
    ''' <param name="defaultValue"></param>
    ''' <returns></returns>
    Private Shared Function GetValueOrDefault(Of valueType)(ByVal Key As String, ByVal defaultValue As valueType) As valueType
        Dim value As valueType

        ' If the key exists, retrieve the value.
        If _isolatedStore.Contains(Key) Then
            value = CType(_isolatedStore(Key), valueType)
            ' Otherwise, use the default value.
        Else
            value = defaultValue
        End If

        Return value
    End Function

    ''' <summary>
    ''' Save the settings.
    ''' </summary>
    Private Shared Sub Save()
        _isolatedStore.Save()
    End Sub

    ''' <summary>
    ''' Property for allowing downloading of feeds only through WiFi
    ''' </summary>
    Public Shared Property WifionlySetting As Boolean
        Get
            Return GetValueOrDefault(Of Boolean)(WifiOnlySettingKeyName, WifiOnlySettingDefault)
        End Get
        Set(ByVal value As Boolean)
            AddOrUpdateValue(WifiOnlySettingKeyName, value)
            Save()
        End Set
    End Property

    ''' <summary>
    ''' Setting that determines whether application 
    ''' </summary>
    Public Shared Property InitialLaunchSetting As Boolean
        Get
            Return GetValueOrDefault(Of Boolean)(InitialLaunchSettingKeyName, InitialLaunchSettingDefault)
        End Get
        Set(ByVal value As Boolean)
            AddOrUpdateValue(InitialLaunchSettingKeyName, value)
            Save()
        End Set
    End Property

    ''' <summary>
    ''' Setting storing the Featured Articles
    ''' </summary>
    Public Shared Property FeaturedArticles As Collection(Of Article)
        Get
            Return GetValueOrDefault(Of Collection(Of Article))(FeaturedArticlesKeyName, FeaturedArticlesDefault)
        End Get
        Set(ByVal value As Collection(Of Article))
            AddOrUpdateValue(FeaturedArticlesKeyName, value)
            Save()
        End Set
    End Property

    ''' <summary>
    ''' Setting storing the Last Updated Time
    ''' </summary>
    Public Shared Property LastUpdatedTime As String
        Get
            Return GetValueOrDefault(Of String)(LastUpdatedTimeKeyName, LastUpdatedDefault)
        End Get
        Set(ByVal value As String)
            AddOrUpdateValue(LastUpdatedTimeKeyName, value)
            Save()
        End Set
    End Property
End Class
