﻿' 
'    Copyright (c) 2011 Microsoft Corporation.  All rights reserved.
'    Use of this sample source code is subject to the terms of the Microsoft license 
'    agreement under which you licensed this sample source code and is provided AS-IS.
'    If you did not accept the terms of the license agreement, you are not authorized 
'    to use this sample source code.  For the terms of the license, please see the 
'    license agreement between you and Microsoft.
'  
'    To see the MSDN article about this app, visit http://go.microsoft.com/fwlink/?LinkId=247592 
'  
'
Imports System.ServiceModel.Syndication

''' <summary>
''' Helper class to obtain the images for feeds.
''' </summary>
Public NotInheritable Class ImageGrabber
    ''' <summary>
    ''' String array holding all of the default image locations
    ''' </summary>
    Private Shared DefaultImages() As String = {"/Images/FeedCastImg0.png", "/Images/FeedCastImg1.png", "/Images/FeedCastImg2.png", "/Images/FeedCastImg3.png", "/Images/FeedCastImg4.png", "/Images/FeedCastImg5.png", "/Images/FeedCastImg6.png", "/Images/FeedCastImg7.png"}

    ''' <summary>
    ''' Determines whether or not the imageURL field of a Feed is empty or not.
    '''  <param name="feed">The feed to be looked at</param>
    ''' <returns>Whether or not the imageURL is empty or not</returns>
    ''' </summary>
    Private Sub New()
    End Sub

    Public Shared Function IfImageExists(ByVal feed As Feed) As Boolean
        Return (feed.ImageURL IsNot Nothing)
    End Function

    ''' <summary>
    ''' Determines whether or not the imageURL field of a SyndicationFeed is empty or not.
    '''<param name="feed">The SyndicationFeed to be looked at</param>
    ''' <returns>Whether or not the imageURL is empty or not</returns>
    ''' </summary>
    Public Shared Function IfImageExists(ByVal synFeed As SyndicationFeed) As Boolean
        Return (synFeed.ImageUrl IsNot Nothing)
    End Function

    ''' <summary>
    ''' Gets the image url for a feed. If there is no usable one, then use a default image.
    ''' </summary>
    ''' <param name="feed">The feed to get the image for</param>
    ''' <param name="synFeed">The SyndicationFeed to obtain data from</param>
    ''' <returns>Whether or not the feed had an obtainable image</returns>
    Public Shared Function GetImage(ByVal feed As Feed, ByVal synFeed As SyndicationFeed) As Boolean
        Dim imageIsOkay As Boolean = False

        If IfImageExists(synFeed) Then
            Dim url As String = synFeed.ImageUrl.ToString

            ' Make sure that an image exists in the right filetype.
            If url.EndsWith("png") OrElse url.EndsWith("jpg") OrElse url.EndsWith("jpeg") Then
                feed.ImageURL = url
                imageIsOkay = True
            End If
        End If
        ' If there's no image, set it to a default one.
        If Not imageIsOkay Then
            feed.ImageURL = GetDefaultImage()
        End If
        Return imageIsOkay
    End Function

    ''' <summary>
    ''' Randomly chooses a default image.
    ''' </summary>
    ''' <returns>Returns the string location to a default image</returns>
    Public Shared Function GetDefaultImage() As String
        Dim random As New Random
        Return (DefaultImages(random.Next(0, 7)))
    End Function
End Class
